_base_ = [
    '../_base_/datasets/icdar2017.py',
    '../_base_/default_runtime.py',
    '../_base_/schedules/schedule_adam_600e.py',
    '_base_panet_resnet50_fpem-ffm.py',
]

default_hooks = dict(checkpoint=dict(type='CheckpointHook', interval=20), )

train_pipeline = [
    dict(type='LoadImageFromFile', color_type='color_ignore_orientation'),
    dict(
        type='LoadOCRAnnotations',
        with_polygon=True,
        with_bbox=True,
        with_label=True,
    ),
    dict(type='ShortScaleAspectJitter', short_size=800, scale_divisor=32),
    dict(type='RandomFlip', prob=0.5, direction='horizontal'),
    dict(type='RandomRotate', max_angle=10),
    dict(type='TextDetRandomCrop', target_size=(800, 800)),
    dict(type='Pad', size=(800, 800)),
    dict(
        type='TorchVisionWrapper',
        op='ColorJitter',
        brightness=32.0 / 255,
        saturation=0.5),
    dict(
        type='PackTextDetInputs',
        meta_keys=('img_path', 'ori_shape', 'img_shape', 'scale_factor'))
]

test_pipeline = [
    dict(type='LoadImageFromFile', color_type='color_ignore_orientation'),
    # TODO Replace with mmcv.RescaleToShort when it's ready
    dict(
        type='ShortScaleAspectJitter',
        short_size=800,
        scale_divisor=1,
        ratio_range=(1.0, 1.0),
        aspect_ratio_range=(1.0, 1.0)),
    dict(
        type='LoadOCRAnnotations',
        with_polygon=True,
        with_bbox=True,
        with_label=True),
    dict(
        type='PackTextDetInputs',
        meta_keys=('img_path', 'ori_shape', 'img_shape', 'scale_factor'))
]
icdar2017_textdet_train = _base_.icdar2017_textdet_train
icdar2017_textdet_test = _base_.icdar2017_textdet_test
# pipeline settings
icdar2017_textdet_train.pipeline = train_pipeline
icdar2017_textdet_test.pipeline = test_pipeline
train_dataloader = dict(
    batch_size=64,
    num_workers=8,
    persistent_workers=True,
    sampler=dict(type='DefaultSampler', shuffle=True),
    dataset=icdar2017_textdet_train)
val_dataloader = dict(
    batch_size=1,
    num_workers=4,
    persistent_workers=True,
    sampler=dict(type='DefaultSampler', shuffle=False),
    dataset=icdar2017_textdet_test)
test_dataloader = val_dataloader

val_evaluator = dict(
    type='HmeanIOUMetric', pred_score_thrs=dict(start=0.3, stop=1, step=0.05))
test_evaluator = val_evaluator

auto_scale_lr = dict(base_batch_size=64)



# Copyright (c) OpenMMLab. All rights reserved.
import os.path as osp
from typing import List, Optional, Tuple, Union

import numpy as np
from mmengine import track_parallel_progress
from scipy.io import loadmat

from mmocr.utils import is_type_list
from ..data_preparer import DATA_PARSERS
from .base import BaseParser


@DATA_PARSERS.register_module()
class SynthTextAnnParser(BaseParser):
    """SynthText Text Detection Annotation Parser.

    Args:
        split (str): The split of the dataset. It is usually set automatically
            and users do not need to set it manually in config file in most
            cases.
        nproc (int): Number of processes to process the data. Defaults to 1.
            It is usually set automatically and users do not need to set it
            manually in config file in most cases.
        separator (str): The separator between each element in a line. Defaults
            to ','.
        ignore (str): The text to be ignored. Defaults to '###'.
        format (str): The format of the annotation. Defaults to
            'x1,y1,x2,y2,x3,y3,x4,trans'.
        encoding (str): The encoding of the annotation file. Defaults to
            'utf-8-sig'.
        remove_strs (List[str], Optional): Used to remove redundant strings in
            the transcription. Defaults to None.
        mode (str, optional): The mode of the box converter. Supported modes
            are 'xywh' and 'xyxy'. Defaults to None.
    """

    def __init__(self,
                 split: str,
                 nproc: int,
                 separator: str = ',',
                 ignore: str = '###',
                 format: str = 'x1,y1,x2,y2,x3,y3,x4,y4,trans',
                 encoding: str = 'utf-8',
                 remove_strs: Optional[List[str]] = None,
                 mode: str = None) -> None:
        self.sep = separator
        self.format = format
        self.encoding = encoding
        self.ignore = ignore
        self.mode = mode
        self.remove_strs = remove_strs
        super().__init__(split=split, nproc=nproc)

    def _trace_boundary(self, char_boxes: List[np.ndarray]) -> np.ndarray:
        """Trace the boundary point of text.

        Args:
            char_boxes (list[ndarray]): The char boxes for one text. Each
                element is 4x2 ndarray.

        Returns:
            ndarray: The boundary point sets with size nx2.
        """
        assert is_type_list(char_boxes, np.ndarray)

        # from top left to to right
        p_top = [box[0:2] for box in char_boxes]
        # from bottom right to bottom left
        p_bottom = [
            char_boxes[idx][[2, 3], :]
            for idx in range(len(char_boxes) - 1, -1, -1)
        ]

        p = p_top + p_bottom

        boundary = np.concatenate(p).astype(int)

        return boundary

    def _match_bbox_char_str(self, bboxes: np.ndarray, char_bboxes: np.ndarray,
                             strs: np.ndarray
                             ) -> Tuple[List[np.ndarray], List[str]]:
        """Match the bboxes, char bboxes, and strs.

        Args:
            bboxes (ndarray): The text boxes of size (2, 4, num_box).
            char_bboxes (ndarray): The char boxes of size (2, 4, num_char_box).
            strs (ndarray): The string of size (num_strs,)

        Returns:
            Tuple(List[ndarray], List[str]): Polygon & word list.
        """
        assert isinstance(bboxes, np.ndarray)
        assert isinstance(char_bboxes, np.ndarray)
        assert isinstance(strs, np.ndarray)
        # bboxes = bboxes.astype(np.int32)
        char_bboxes = char_bboxes.astype(np.int32)

        if len(char_bboxes.shape) == 2:
            char_bboxes = np.expand_dims(char_bboxes, axis=2)
        char_bboxes = np.transpose(char_bboxes, (2, 1, 0))
        num_boxes = 1 if len(bboxes.shape) == 2 else bboxes.shape[-1]

        poly_charbox_list = [[] for _ in range(num_boxes)]

        words = []
        for line in strs:
            words += line.split()
        words_len = [len(w) for w in words]
        words_end_inx = np.cumsum(words_len)
        start_inx = 0
        for word_inx, end_inx in enumerate(words_end_inx):
            for char_inx in range(start_inx, end_inx):
                poly_charbox_list[word_inx].append(char_bboxes[char_inx])
            start_inx = end_inx

        for box_inx in range(num_boxes):
            assert len(poly_charbox_list[box_inx]) > 0

        poly_boundary_list = []
        for item in poly_charbox_list:
            boundary = np.ndarray((0, 2))
            if len(item) > 0:
                boundary = self._trace_boundary(item)
            poly_boundary_list.append(boundary)

        return poly_boundary_list, words

    def parse_files(self, img_paths: Union[List[str], str],
                    ann_paths: Union[List[str], str]) -> List[Tuple]:
        """Convert annotations to MMOCR format.

        Args:
            img_paths (str or list[str]): the list of image paths or the
                directory of the images.
            ann_paths (str or list[str]): the list of annotation paths or the
                path of the annotation file which contains all the annotations.

        Returns:
            List[Tuple]: A list of a tuple of (image_path, instances).

            - img_path (str): The path of image file, which can be read
              directly by opencv.
            - instance: instance is a list of dict containing parsed
              annotations, which should contain the following keys:

              - 'poly' or 'box' (textdet or textspotting)
              - 'text' (textspotting or textrecog)
              - 'ignore' (all task)
        """
        assert isinstance(ann_paths, str)
        gt = loadmat(ann_paths)
        self.img_dir = img_paths
        samples = track_parallel_progress(
            self.parse_file,
            list(
                zip(gt['imnames'][0], gt['wordBB'][0], gt['charBB'][0],
                    gt['txt'][0])),
            nproc=self.nproc)
        return samples

    def parse_file(self, annotation: Tuple) -> Tuple:
        """Parse single annotation."""
        img_file, wordBB, charBB, txt = annotation
        polys_list, word_list = self._match_bbox_char_str(wordBB, charBB, txt)

        instances = list()
        for poly, word in zip(polys_list, word_list):
            instances.append(
                dict(poly=poly.flatten().tolist(), text=word, ignore=False))
        return osp.join(self.img_dir, img_file[0]), instances



# Copyright (c) OpenMMLab. All rights reserved.
import argparse
import os
import os.path as osp

import mmcv
import mmengine

from mmocr.utils import dump_ocr_data


def collect_files(img_dir, gt_dir):
    """Collect all images and their corresponding groundtruth files.

    Args:
        img_dir (str): The image directory
        gt_dir (str): The groundtruth directory

    Returns:
        files (list): The list of tuples (img_file, groundtruth_file)
    """
    assert isinstance(img_dir, str)
    assert img_dir
    assert isinstance(gt_dir, str)
    assert gt_dir

    ann_list, imgs_list = [], []
    for img_file in os.listdir(img_dir):
        ann_file = 'gt_' + str(int(img_file[2:6])) + '.txt'
        ann_list.append(osp.join(gt_dir, ann_file))
        imgs_list.append(osp.join(img_dir, img_file))

    files = list(zip(imgs_list, ann_list))
    assert len(files), f'No images found in {img_dir}'
    print(f'Loaded {len(files)} images from {img_dir}')

    return files


def collect_annotations(files, nproc=1):
    """Collect the annotation information.

    Args:
        files (list): The list of tuples (image_file, groundtruth_file)
        nproc (int): The number of process to collect annotations

    Returns:
        images (list): The list of image information dicts
    """
    assert isinstance(files, list)
    assert isinstance(nproc, int)

    if nproc > 1:
        images = mmengine.track_parallel_progress(
            load_img_info, files, nproc=nproc)
    else:
        images = mmengine.track_progress(load_img_info, files)

    return images


def load_img_info(files):
    """Load the information of one image.

    Args:
        files (tuple): The tuple of (img_file, groundtruth_file)

    Returns:
        img_info (dict): The dict of the img and annotation information
    """
    assert isinstance(files, tuple)

    img_file, gt_file = files
    assert int(osp.basename(gt_file)[3:-4]) == int(
        osp.basename(img_file)[2:-4])
    # read imgs while ignoring orientations
    img = mmcv.imread(img_file, 'unchanged')

    img_info = dict(
        file_name=osp.basename(img_file),
        height=img.shape[0],
        width=img.shape[1],
        segm_file=osp.basename(gt_file))

    if osp.splitext(gt_file)[1] == '.txt':
        img_info = load_txt_info(gt_file, img_info)
    else:
        raise NotImplementedError

    return img_info


def load_txt_info(gt_file, img_info):
    """Collect the annotation information.

    The annotation format is as the following:
    x1,y1,x2,y2,x3,y3,x4,y4,text
    118,15,147,15,148,46,118,46,LƯỢNG
    149,9,165,9,165,43,150,43,TỐT
    167,9,180,9,179,43,167,42,ĐỂ
    181,12,193,12,193,43,181,43,CÓ
    195,13,215,14,215,46,196,46,VIỆC
    217,13,237,14,239,47,217,46,LÀM,

    Args:
        gt_file (str): The path to ground-truth
        img_info (dict): The dict of the img and annotation information

    Returns:
        img_info (dict): The dict of the img and annotation information
    """

    with open(gt_file, encoding='utf-8') as f:
        anno_info = []
        for line in f:
            line = line.strip('\n')
            ann = line.split(',')
            bbox = ann[0:8]
            word = line[len(','.join(bbox)) + 1:]
            bbox = [int(coord) for coord in bbox]
            segmentation = bbox
            x_min = min(bbox[0], bbox[2], bbox[4], bbox[6])
            x_max = max(bbox[0], bbox[2], bbox[4], bbox[6])
            y_min = min(bbox[1], bbox[3], bbox[5], bbox[7])
            y_max = max(bbox[1], bbox[3], bbox[5], bbox[7])
            w = x_max - x_min
            h = y_max - y_min
            bbox = [x_min, y_min, w, h]
            iscrowd = 1 if word == '###' else 0

            anno = dict(
                iscrowd=iscrowd,
                category_id=1,
                bbox=bbox,
                area=w * h,
                segmentation=[segmentation])
            anno_info.append(anno)

    img_info.update(anno_info=anno_info)

    return img_info


def parse_args():
    parser = argparse.ArgumentParser(
        description='Generate training and test set of VinText ')
    parser.add_argument('root_path', help='Root dir path of VinText')
    parser.add_argument(
        '--nproc', default=1, type=int, help='Number of processes')
    args = parser.parse_args()
    return args


def main():
    args = parse_args()
    root_path = args.root_path
    for split in ['training', 'test', 'unseen_test']:
        print(f'Processing {split} set...')
        with mmengine.Timer(
                print_tmpl='It takes {}s to convert VinText annotation'):
            files = collect_files(
                osp.join(root_path, 'imgs', split),
                osp.join(root_path, 'annotations'))
            image_infos = collect_annotations(files, nproc=args.nproc)
            dump_ocr_data(image_infos,
                          osp.join(root_path, 'instances_' + split + '.json'),
                          'textdet')


if __name__ == '__main__':
    main()



# Copyright (c) OpenMMLab. All rights reserved.
from abc import ABCMeta, abstractmethod
from typing import Dict, Sequence, Tuple, Union

import torch
from torch import nn

from mmocr.registry import MODELS
from mmocr.utils.typing_utils import DetSampleList

INPUT_TYPES = Union[torch.Tensor, Sequence[torch.Tensor], Dict]


@MODELS.register_module()
class BaseTextDetModuleLoss(nn.Module, metaclass=ABCMeta):
    r"""Base class for text detection module loss.
    """

    def __init__(self) -> None:
        super().__init__()

    @abstractmethod
    def forward(self,
                inputs: INPUT_TYPES,
                data_samples: DetSampleList = None) -> Dict:
        """Calculates losses from a batch of inputs and data samples. Returns a
        dict of losses.

        Args:
            inputs (Tensor or list[Tensor] or dict): The raw tensor outputs
                from the model.
            data_samples (list(TextDetDataSample)): Datasamples containing
                ground truth data.

        Returns:
            dict: A dict of losses.
        """
        pass

    @abstractmethod
    def get_targets(self, data_samples: DetSampleList) -> Tuple:
        """Generates loss targets from data samples. Returns a tuple of target
        tensors.

        Args:
            data_samples (list(TextDetDataSample)): Ground truth data samples.

        Returns:
            tuple: A tuple of target tensors.
        """
        pass



# Copyright (c) OpenMMLab. All rights reserved.
import warnings
from typing import Dict, Optional, Sequence, Tuple, Union

import mmengine
import torch
from mmengine.structures import LabelData

from mmocr.models.common.dictionary import Dictionary
from mmocr.registry import TASK_UTILS
from mmocr.structures import TextRecogDataSample


class BaseTextRecogPostprocessor:
    """Base text recognition postprocessor.

    Args:
        dictionary (dict or :obj:`Dictionary`): The config for `Dictionary` or
            the instance of `Dictionary`.
        max_seq_len (int): max_seq_len (int): Maximum sequence length. The
            sequence is usually generated from decoder. Defaults to 40.
        ignore_chars (list[str]): A list of characters to be ignored from the
            final results. Postprocessor will skip over these characters when
            converting raw indexes to characters. Apart from single characters,
            each item can be one of the following reversed keywords: 'padding',
            'end' and 'unknown', which refer to their corresponding special
            tokens in the dictionary.
    """

    def __init__(self,
                 dictionary: Union[Dictionary, Dict],
                 max_seq_len: int = 40,
                 ignore_chars: Sequence[str] = ['padding'],
                 **kwargs) -> None:

        if isinstance(dictionary, dict):
            self.dictionary = TASK_UTILS.build(dictionary)
        elif isinstance(dictionary, Dictionary):
            self.dictionary = dictionary
        else:
            raise TypeError(
                'The type of dictionary should be `Dictionary` or dict, '
                f'but got {type(dictionary)}')
        self.max_seq_len = max_seq_len

        mapping_table = {
            'padding': self.dictionary.padding_idx,
            'end': self.dictionary.end_idx,
            'unknown': self.dictionary.unknown_idx,
        }
        if not mmengine.is_list_of(ignore_chars, str):
            raise TypeError('ignore_chars must be list of str')
        ignore_indexes = list()
        for ignore_char in ignore_chars:
            index = mapping_table.get(
                ignore_char,
                self.dictionary.char2idx(ignore_char, strict=False))
            if index is None or (index == self.dictionary.unknown_idx
                                 and ignore_char != 'unknown'):
                warnings.warn(
                    f'{ignore_char} does not exist in the dictionary',
                    UserWarning)
                continue
            ignore_indexes.append(index)
        self.ignore_indexes = ignore_indexes

    def get_single_prediction(
        self,
        probs: torch.Tensor,
        data_sample: Optional[TextRecogDataSample] = None,
    ) -> Tuple[Sequence[int], Sequence[float]]:
        """Convert the output probabilities of a single image to index and
        score.

        Args:
           probs (torch.Tensor): Character probabilities with shape
                :math:`(T, C)`.
           data_sample (TextRecogDataSample): Datasample of an image.

        Returns:
            tuple(list[int], list[float]): Index and scores per-character.
        """
        raise NotImplementedError

    def __call__(
        self, probs: torch.Tensor, data_samples: Sequence[TextRecogDataSample]
    ) -> Sequence[TextRecogDataSample]:
        """Convert outputs to strings and scores.

        Args:
            probs (torch.Tensor): Batched character probabilities, the model's
                softmaxed output in size: :math:`(N, T, C)`.
            data_samples (list[TextRecogDataSample]): The list of
                TextRecogDataSample.

        Returns:
            list(TextRecogDataSample): The list of TextRecogDataSample. It
            usually contain ``pred_text`` information.
        """
        batch_size = probs.size(0)

        for idx in range(batch_size):
            index, score = self.get_single_prediction(probs[idx, :, :],
                                                      data_samples[idx])
            text = self.dictionary.idx2str(index)
            pred_text = LabelData()
            pred_text.score = score
            pred_text.item = text
            data_samples[idx].pred_text = pred_text
        return data_samples



# Copyright (c) OpenMMLab. All rights reserved.
from .config_generators import *  # noqa
from .data_preparer import DatasetPreparer
from .dumpers import *  # noqa
from .gatherers import *  # noqa
from .obtainers import *  # noqa
from .packers import *  # noqa
from .parsers import *  # noqa

__all__ = ['DatasetPreparer']



_base_ = [
    '../_base_/default_runtime.py',
    '../_base_/datasets/wildreceipt-openset.py',
    '../_base_/schedules/schedule_adam_60e.py',
    '_base_sdmgr_novisual.py',
]

node_num_classes = 4  # 4 classes: bg, key, value and other
edge_num_classes = 2  # edge connectivity
key_node_idx = 1
value_node_idx = 2

model = dict(
    type='SDMGR',
    kie_head=dict(
        num_classes=node_num_classes,
        postprocessor=dict(
            link_type='one-to-many',
            key_node_idx=key_node_idx,
            value_node_idx=value_node_idx)),
)

test_pipeline = [
    dict(
        type='LoadKIEAnnotations',
        key_node_idx=key_node_idx,
        value_node_idx=value_node_idx),  # Keep key->value edges for evaluation
    dict(type='Resize', scale=(1024, 512), keep_ratio=True),
    dict(type='PackKIEInputs'),
]

wildreceipt_openset_train = _base_.wildreceipt_openset_train
wildreceipt_openset_train.pipeline = _base_.train_pipeline
wildreceipt_openset_test = _base_.wildreceipt_openset_test
wildreceipt_openset_test.pipeline = test_pipeline

train_dataloader = dict(
    batch_size=4,
    num_workers=1,
    persistent_workers=True,
    sampler=dict(type='DefaultSampler', shuffle=True),
    dataset=wildreceipt_openset_train)
val_dataloader = dict(
    batch_size=1,
    num_workers=1,
    persistent_workers=True,
    sampler=dict(type='DefaultSampler', shuffle=False),
    dataset=wildreceipt_openset_test)
test_dataloader = val_dataloader

val_evaluator = [
    dict(
        type='F1Metric',
        prefix='node',
        key='labels',
        mode=['micro', 'macro'],
        num_classes=node_num_classes,
        cared_classes=[key_node_idx, value_node_idx]),
    dict(
        type='F1Metric',
        prefix='edge',
        mode='micro',
        key='edge_labels',
        cared_classes=[1],  # Collapse to binary F1 score
        num_classes=edge_num_classes)
]
test_evaluator = val_evaluator

visualizer = dict(
    type='KIELocalVisualizer', name='visualizer', is_openset=True)
auto_scale_lr = dict(base_batch_size=4)



# Copyright (c) OpenMMLab. All rights reserved.
import argparse
import math
import os.path as osp
from functools import partial

import mmcv
import mmengine

from mmocr.utils import dump_ocr_data


def parse_args():
    parser = argparse.ArgumentParser(
        description='Generate training and validation set of LSVT ')
    parser.add_argument('root_path', help='Root dir path of LSVT')
    parser.add_argument(
        '--val-ratio', help='Split ratio for val set', default=0.0, type=float)
    parser.add_argument(
        '--nproc', default=1, type=int, help='Number of processes')
    parser.add_argument(
        '--preserve-vertical',
        help='Preserve samples containing vertical texts',
        action='store_true')
    args = parser.parse_args()
    return args


def process_img(args, dst_image_root, ignore_image_root, preserve_vertical,
                split):
    # Dirty hack for multi-processing
    img_idx, img_info, anns = args
    src_img = mmcv.imread(img_info['file_name'])
    img_info = []
    for ann_idx, ann in enumerate(anns):
        segmentation = []
        for x, y in ann['points']:
            segmentation.append(max(0, x))
            segmentation.append(max(0, y))
        xs, ys = segmentation[::2], segmentation[1::2]
        x, y = min(xs), min(ys)
        w, h = max(xs) - x, max(ys) - y
        text_label = ann['transcription']

        dst_img = src_img[y:y + h, x:x + w]
        dst_img_name = f'img_{img_idx}_{ann_idx}.jpg'

        if not preserve_vertical and h / w > 2 and split == 'train':
            dst_img_path = osp.join(ignore_image_root, dst_img_name)
            mmcv.imwrite(dst_img, dst_img_path)
            continue

        dst_img_path = osp.join(dst_image_root, dst_img_name)
        mmcv.imwrite(dst_img, dst_img_path)

        img_info.append({
            'file_name': dst_img_name,
            'anno_info': [{
                'text': text_label
            }]
        })

    return img_info


def convert_lsvt(root_path,
                 split,
                 ratio,
                 preserve_vertical,
                 nproc,
                 img_start_idx=0):
    """Collect the annotation information and crop the images.

    The annotation format is as the following:
    [
        {'gt_1234': # 'gt_1234' is file name
            [
                {
                    'transcription': '一站式购物中心',
                    'points': [[45, 272], [215, 273], [212, 296], [45, 290]]
                    'illegibility': False
                }, ...
            ]
        }
    ]


    Args:
        root_path (str): The root path of the dataset
        split (str): The split of dataset. Namely: training or val
        ratio (float): Split ratio for val set
        preserve_vertical (bool): Whether to preserve vertical texts
        nproc (int): The number of process to collect annotations
        img_start_idx (int): Index of start image

    Returns:
        img_info (dict): The dict of the img and annotation information
    """

    annotation_path = osp.join(root_path, 'annotations/train_full_labels.json')
    if not osp.exists(annotation_path):
        raise Exception(
            f'{annotation_path} not exists, please check and try again.')

    annotation = mmengine.load(annotation_path)
    # outputs
    dst_label_file = osp.join(root_path, f'{split}_label.json')
    dst_image_root = osp.join(root_path, 'crops', split)
    ignore_image_root = osp.join(root_path, 'ignores', split)
    src_image_root = osp.join(root_path, 'imgs')
    mmengine.mkdir_or_exist(dst_image_root)
    mmengine.mkdir_or_exist(ignore_image_root)

    process_img_with_path = partial(
        process_img,
        dst_image_root=dst_image_root,
        ignore_image_root=ignore_image_root,
        preserve_vertical=preserve_vertical,
        split=split)

    img_prefixes = annotation.keys()

    trn_files, val_files = [], []
    if ratio > 0:
        for i, file in enumerate(img_prefixes):
            if i % math.floor(1 / ratio):
                trn_files.append(file)
            else:
                val_files.append(file)
    else:
        trn_files, val_files = img_prefixes, []
    print(f'training #{len(trn_files)}, val #{len(val_files)}')

    if split == 'train':
        img_prefixes = trn_files
    elif split == 'val':
        img_prefixes = val_files
    else:
        raise NotImplementedError

    tasks = []
    idx = 0
    for img_idx, prefix in enumerate(img_prefixes):
        img_file = osp.join(src_image_root, prefix + '.jpg')
        img_info = {'file_name': img_file}
        # Skip not exist images
        if not osp.exists(img_file):
            continue
        tasks.append((img_idx + img_start_idx, img_info, annotation[prefix]))
        idx = idx + 1

    labels_list = mmengine.track_parallel_progress(
        process_img_with_path, tasks, keep_order=True, nproc=nproc)
    final_labels = []
    for label_list in labels_list:
        final_labels += label_list

    dump_ocr_data(final_labels, dst_label_file, 'textrecog')

    return idx


def main():
    args = parse_args()
    root_path = args.root_path
    print('Processing training set...')
    num_train_imgs = convert_lsvt(
        root_path=root_path,
        split='train',
        ratio=args.val_ratio,
        preserve_vertical=args.preserve_vertical,
        nproc=args.nproc)
    if args.val_ratio > 0:
        print('Processing validation set...')
        convert_lsvt(
            root_path=root_path,
            split='val',
            ratio=args.val_ratio,
            preserve_vertical=args.preserve_vertical,
            nproc=args.nproc,
            img_start_idx=num_train_imgs)
    print('Finish')


if __name__ == '__main__':
    main()



# Copyright (c) OpenMMLab. All rights reserved.
from .kie_data_sample import KIEDataSample
from .textdet_data_sample import TextDetDataSample
from .textrecog_data_sample import TextRecogDataSample
from .textspotting_data_sample import TextSpottingDataSample

__all__ = [
    'TextDetDataSample', 'TextRecogDataSample', 'KIEDataSample',
    'TextSpottingDataSample'
]



# Copyright (c) OpenMMLab. All rights reserved.
import sys
from typing import Optional, Sequence, Tuple, Union

import cv2
import numpy as np
import torch
from mmengine.logging import MMLogger
from shapely.geometry import Polygon

from mmocr.utils.polygon_utils import offset_polygon
from .base import BaseTextDetModuleLoss


class SegBasedModuleLoss(BaseTextDetModuleLoss):
    """Base class for the module loss of segmentation-based text detection
    algorithms with some handy utilities."""

    def _generate_kernels(
        self,
        img_size: Tuple[int, int],
        text_polys: Sequence[np.ndarray],
        shrink_ratio: float,
        max_shrink_dist: Union[float, int] = sys.maxsize,
        ignore_flags: Optional[torch.Tensor] = None
    ) -> Tuple[np.ndarray, np.ndarray]:
        """Generate text instance kernels according to a shrink ratio.

        Args:
            img_size (tuple(int, int)): The image size of (height, width).
            text_polys (Sequence[np.ndarray]): 2D array of text polygons.
            shrink_ratio (float or int): The shrink ratio of kernel.
            max_shrink_dist (float or int): The maximum shrinking distance.
            ignore_flags (torch.BoolTensor, optional): Indicate whether the
                corresponding text polygon is ignored. Defaults to None.

        Returns:
            tuple(ndarray, ndarray): The text instance kernels of shape
                (height, width) and updated ignorance flags.
        """
        assert isinstance(img_size, tuple)
        assert isinstance(shrink_ratio, (float, int))

        logger: MMLogger = MMLogger.get_current_instance()

        h, w = img_size
        text_kernel = np.zeros((h, w), dtype=np.float32)

        for text_ind, poly in enumerate(text_polys):
            if ignore_flags is not None and ignore_flags[text_ind]:
                continue
            poly = poly.reshape(-1, 2).astype(np.int32)
            poly_obj = Polygon(poly)
            area = poly_obj.area
            peri = poly_obj.length
            distance = min(
                int(area * (1 - shrink_ratio * shrink_ratio) / (peri + 0.001) +
                    0.5), max_shrink_dist)
            shrunk_poly = offset_polygon(poly, -distance)

            if len(shrunk_poly) == 0:
                if ignore_flags is not None:
                    ignore_flags[text_ind] = True
                continue

            try:
                shrunk_poly = shrunk_poly.reshape(-1, 2)
            except Exception as e:
                logger.info(f'{shrunk_poly} with error {e}')
                if ignore_flags is not None:
                    ignore_flags[text_ind] = True
                continue

            cv2.fillPoly(text_kernel, [shrunk_poly.astype(np.int32)],
                         text_ind + 1)

        return text_kernel, ignore_flags

    def _generate_effective_mask(self, mask_size: Tuple[int, int],
                                 ignored_polygons: Sequence[np.ndarray]
                                 ) -> np.ndarray:
        """Generate effective mask by setting the invalid regions to 0 and 1
        otherwise.

        Args:
            mask_size (tuple(int, int)): The mask size.
            ignored_polygons (Sequence[ndarray]): 2-d array, representing all
                the ignored polygons of the text region.

        Returns:
            mask (ndarray): The effective mask of shape (height, width).
        """

        mask = np.ones(mask_size, dtype=np.uint8)

        for poly in ignored_polygons:
            instance = poly.astype(np.int32).reshape(1, -1, 2)
            cv2.fillPoly(mask, instance, 0)

        return mask



# training schedule for 1x
_base_ = [
    '../_base_/datasets/mjsynth.py',
    '../_base_/datasets/cute80.py',
    '../_base_/datasets/iiit5k.py',
    '../_base_/datasets/svt.py',
    '../_base_/datasets/svtp.py',
    '../_base_/datasets/icdar2013.py',
    '../_base_/datasets/icdar2015.py',
    '../_base_/default_runtime.py',
    '../_base_/schedules/schedule_adadelta_5e.py',
    '_base_crnn_mini-vgg.py',
]
# dataset settings
train_list = [_base_.mjsynth_textrecog_train]
test_list = [
    _base_.cute80_textrecog_test, _base_.iiit5k_textrecog_test,
    _base_.svt_textrecog_test, _base_.svtp_textrecog_test,
    _base_.icdar2013_textrecog_test, _base_.icdar2015_textrecog_test
]

default_hooks = dict(logger=dict(type='LoggerHook', interval=50), )
train_dataloader = dict(
    batch_size=64,
    num_workers=24,
    persistent_workers=True,
    sampler=dict(type='DefaultSampler', shuffle=True),
    dataset=dict(
        type='ConcatDataset',
        datasets=train_list,
        pipeline=_base_.train_pipeline))
test_dataloader = dict(
    batch_size=1,
    num_workers=4,
    persistent_workers=True,
    drop_last=False,
    sampler=dict(type='DefaultSampler', shuffle=False),
    dataset=dict(
        type='ConcatDataset',
        datasets=test_list,
        pipeline=_base_.test_pipeline))
val_dataloader = test_dataloader

val_evaluator = dict(
    dataset_prefixes=['CUTE80', 'IIIT5K', 'SVT', 'SVTP', 'IC13', 'IC15'])
test_evaluator = val_evaluator

auto_scale_lr = dict(base_batch_size=64 * 4)



# Copyright (c) OpenMMLab. All rights reserved.
import json
import warnings
from typing import Dict, Tuple

from mmocr.registry import TASK_UTILS
from mmocr.utils.string_utils import StringStripper


@TASK_UTILS.register_module()
class LineStrParser:
    """Parse string of one line in annotation file to dict format.

    Args:
        keys (list[str]): Keys in result dict. Defaults to
            ['filename', 'text'].
        keys_idx (list[int]): Value index in sub-string list for each key
            above. Defaults to [0, 1].
        separator (str): Separator to separate string to list of sub-string.
            Defaults to ' '.
    """

    def __init__(self,
                 keys: Tuple[str, str] = ['filename', 'text'],
                 keys_idx: Tuple[int, int] = [0, 1],
                 separator: str = ' ',
                 **kwargs):
        assert isinstance(keys, list)
        assert isinstance(keys_idx, list)
        assert isinstance(separator, str)
        assert len(keys) > 0
        assert len(keys) == len(keys_idx)
        self.keys = keys
        self.keys_idx = keys_idx
        self.separator = separator
        self.strip_cls = StringStripper(**kwargs)

    def __call__(self, in_str: str) -> Dict:
        line_str = self.strip_cls(in_str)
        if len(line_str.split(' ')) > 2:
            msg = 'More than two blank spaces were detected. '
            msg += 'Please use LineJsonParser to handle '
            msg += 'annotations with blanks. '
            msg += 'Check Doc '
            msg += 'https://mmocr.readthedocs.io/en/latest/'
            msg += 'tutorials/blank_recog.html '
            msg += 'for details.'
            warnings.warn(msg, UserWarning)
        line_str = line_str.split(self.separator)
        if len(line_str) <= max(self.keys_idx):
            raise ValueError(
                f'key index: {max(self.keys_idx)} out of range: {line_str}')

        line_info = {}
        for i, key in enumerate(self.keys):
            line_info[key] = line_str[self.keys_idx[i]]
        return line_info


@TASK_UTILS.register_module()
class LineJsonParser:
    """Parse json-string of one line in annotation file to dict format.

    Args:
        keys (list[str]): Keys in both json-string and result dict. Defaults
            to ['filename', 'text'].
    """

    def __init__(self, keys: Tuple[str, str] = ['filename', 'text']) -> None:
        assert isinstance(keys, list)
        assert len(keys) > 0
        self.keys = keys

    def __call__(self, in_str: str) -> Dict:
        line_json_obj = json.loads(in_str)
        line_info = {}
        for key in self.keys:
            if key not in line_json_obj:
                raise Exception(f'key {key} not in line json {line_json_obj}')
            line_info[key] = line_json_obj[key]

        return line_info



# Copyright (c) OpenMMLab. All rights reserved.
from mmocr.registry import MODELS
from .single_stage_text_detector import SingleStageTextDetector


@MODELS.register_module()
class PANet(SingleStageTextDetector):
    """The class for implementing PANet text detector:

    Efficient and Accurate Arbitrary-Shaped Text Detection with Pixel
    Aggregation Network [https://arxiv.org/abs/1908.05900].
    """



# Copyright (c) OpenMMLab. All rights reserved.
from typing import List, Optional, Tuple

from mmocr.registry import DATA_PARSERS
from mmocr.utils import bbox2poly
from .base import BaseParser


@DATA_PARSERS.register_module()
class SROIETextDetAnnParser(BaseParser):
    """SROIE Txt Format Text Detection Annotation Parser.

    The original annotation format of this dataset is stored in txt files,
    which is formed as the following format:
        x1, y1, x2, y2, x3, y3, x4, y4, transcription

    Args:
        separator (str): The separator between each element in a line. Defaults
            to ','.
        ignore (str): The text to be ignored. Defaults to '###'.
        format (str): The format of the annotation. Defaults to
            'x1,y1,x2,y2,x3,y3,x4,trans'.
        encoding (str): The encoding of the annotation file. Defaults to
            'utf-8-sig'.
        nproc (int): The number of processes to parse the annotation. Defaults
            to 1.
        remove_strs (List[str], Optional): Used to remove redundant strings in
            the transcription. Defaults to None.
        mode (str, optional): The mode of the box converter. Supported modes
            are 'xywh' and 'xyxy'. Defaults to None.
    """

    def __init__(self,
                 split: str,
                 separator: str = ',',
                 ignore: str = '###',
                 format: str = 'x1,y1,x2,y2,x3,y3,x4,y4,trans',
                 encoding: str = 'utf-8-sig',
                 nproc: int = 1,
                 remove_strs: Optional[List[str]] = None,
                 mode: str = None) -> None:
        self.sep = separator
        self.format = format
        self.encoding = encoding
        self.ignore = ignore
        self.mode = mode
        self.remove_strs = remove_strs
        super().__init__(nproc=nproc, split=split)

    def parse_file(self, img_path: str, ann_path: str) -> Tuple:
        """Parse single annotation."""
        instances = list()
        try:
            # there might be some illegal symbols in the annotation
            # which cannot be parsed by loader
            for anno in self.loader(ann_path, self.sep, self.format,
                                    self.encoding):
                anno = list(anno.values())
                if self.remove_strs is not None:
                    for strs in self.remove_strs:
                        for i in range(len(anno)):
                            if strs in anno[i]:
                                anno[i] = anno[i].replace(strs, '')
                poly = list(map(float, anno[0:-1]))
                if self.mode is not None:
                    poly = bbox2poly(poly, self.mode)
                    poly = poly.tolist()
                text = anno[-1]
                instances.append(
                    dict(poly=poly, text=text, ignore=text == self.ignore))
        except Exception:
            pass

        return img_path, instances



# Copyright (c) OpenMMLab. All rights reserved.
from typing import Optional, Sequence, Union

import mmcv
import numpy as np
import torch

from mmocr.registry import VISUALIZERS
from mmocr.structures import TextDetDataSample
from mmocr.utils.polygon_utils import poly2bbox
from .base_visualizer import BaseLocalVisualizer


@VISUALIZERS.register_module()
class TextSpottingLocalVisualizer(BaseLocalVisualizer):

    def _draw_instances(
        self,
        image: np.ndarray,
        bboxes: Union[np.ndarray, torch.Tensor],
        polygons: Sequence[np.ndarray],
        texts: Sequence[str],
    ) -> np.ndarray:
        """Draw instances on image.

        Args:
            image (np.ndarray): The origin image to draw. The format
                should be RGB.
            bboxes (np.ndarray, torch.Tensor): The bboxes to draw. The shape of
                bboxes should be (N, 4), where N is the number of texts.
            polygons (Sequence[np.ndarray]): The polygons to draw. The length
                of polygons should be the same as the number of bboxes.
            edge_labels (np.ndarray, torch.Tensor): The edge labels to draw.
                The shape of edge_labels should be (N, N), where N is the
                number of texts.
            texts (Sequence[str]): The texts to draw. The length of texts
                should be the same as the number of bboxes.
            class_names (dict): The class names for bbox labels.
            is_openset (bool): Whether the dataset is openset. Default: False.

        Returns:
            np.ndarray: The image with instances drawn.
        """
        img_shape = image.shape[:2]
        empty_shape = (img_shape[0], img_shape[1], 3)
        text_image = np.full(empty_shape, 255, dtype=np.uint8)
        if texts:
            text_image = self.get_labels_image(
                text_image,
                labels=texts,
                bboxes=bboxes,
                font_families=self.font_families,
                font_properties=self.font_properties)
        if polygons:
            polygons = [polygon.reshape(-1, 2) for polygon in polygons]
            image = self.get_polygons_image(
                image, polygons, filling=True, colors=self.PALETTE)
            text_image = self.get_polygons_image(
                text_image, polygons, colors=self.PALETTE)
        elif len(bboxes) > 0:
            image = self.get_bboxes_image(
                image, bboxes, filling=True, colors=self.PALETTE)
            text_image = self.get_bboxes_image(
                text_image, bboxes, colors=self.PALETTE)
        return np.concatenate([image, text_image], axis=1)

    def add_datasample(self,
                       name: str,
                       image: np.ndarray,
                       data_sample: Optional['TextDetDataSample'] = None,
                       draw_gt: bool = True,
                       draw_pred: bool = True,
                       show: bool = False,
                       wait_time: int = 0,
                       pred_score_thr: float = 0.5,
                       out_file: Optional[str] = None,
                       step: int = 0) -> None:
        """Draw datasample and save to all backends.

        - If GT and prediction are plotted at the same time, they are
        displayed in a stitched image where the left image is the
        ground truth and the right image is the prediction.
        - If ``show`` is True, all storage backends are ignored, and
        the images will be displayed in a local window.
        - If ``out_file`` is specified, the drawn image will be
        saved to ``out_file``. This is usually used when the display
        is not available.

        Args:
            name (str): The image identifier.
            image (np.ndarray): The image to draw.
            data_sample (:obj:`TextSpottingDataSample`, optional):
                TextDetDataSample which contains gt and prediction. Defaults
                    to None.
            draw_gt (bool): Whether to draw GT TextDetDataSample.
                Defaults to True.
            draw_pred (bool): Whether to draw Predicted TextDetDataSample.
                Defaults to True.
            show (bool): Whether to display the drawn image. Default to False.
            wait_time (float): The interval of show (s). Defaults to 0.
            out_file (str): Path to output file. Defaults to None.
            pred_score_thr (float): The threshold to visualize the bboxes
                and masks. Defaults to 0.3.
            step (int): Global step value to record. Defaults to 0.
        """
        cat_images = []

        if data_sample is not None:
            if draw_gt and 'gt_instances' in data_sample:
                gt_bboxes = data_sample.gt_instances.get('bboxes', None)
                gt_texts = data_sample.gt_instances.texts
                gt_polygons = data_sample.gt_instances.get('polygons', None)
                gt_img_data = self._draw_instances(image, gt_bboxes,
                                                   gt_polygons, gt_texts)
                cat_images.append(gt_img_data)

            if draw_pred and 'pred_instances' in data_sample:
                pred_instances = data_sample.pred_instances
                pred_instances = pred_instances[
                    pred_instances.scores > pred_score_thr].cpu().numpy()
                pred_bboxes = pred_instances.get('bboxes', None)
                pred_texts = pred_instances.texts
                pred_polygons = pred_instances.get('polygons', None)
                if pred_bboxes is None:
                    pred_bboxes = [poly2bbox(poly) for poly in pred_polygons]
                    pred_bboxes = np.array(pred_bboxes)
                pred_img_data = self._draw_instances(image, pred_bboxes,
                                                     pred_polygons, pred_texts)
                cat_images.append(pred_img_data)

        cat_images = self._cat_image(cat_images, axis=0)
        if cat_images is None:
            cat_images = image

        if show:
            self.show(cat_images, win_name=name, wait_time=wait_time)
        else:
            self.add_image(name, cat_images, step)

        if out_file is not None:
            mmcv.imwrite(cat_images[..., ::-1], out_file)

        self.set_image(cat_images)
        return self.get_image()



# optimizer
optim_wrapper = dict(
    type='OptimWrapper',
    optimizer=dict(type='SGD', lr=0.007, momentum=0.9, weight_decay=0.0001))

train_cfg = dict(type='IterBasedTrainLoop', max_iters=100000)
test_cfg = None
val_cfg = None
# learning policy
param_scheduler = [
    dict(type='PolyLR', power=0.9, eta_min=1e-7, by_epoch=False, end=100000),
]



# Copyright (c) OpenMMLab. All rights reserved.
import numpy as np
from scipy.special import comb as n_over_k

from mmocr.utils.typing_utils import ArrayLike


def bezier_coefficient(n, t, k):
    return t**k * (1 - t)**(n - k) * n_over_k(n, k)


def bezier_coefficients(time, point_num, ratios):
    return [[bezier_coefficient(time, ratio, num) for num in range(point_num)]
            for ratio in ratios]


def linear_interpolation(point1: np.ndarray,
                         point2: np.ndarray,
                         number: int = 2) -> np.ndarray:
    t = np.linspace(0, 1, number + 2).reshape(-1, 1)
    return point1 + (point2 - point1) * t


def curve2bezier(curve: ArrayLike):
    curve = np.array(curve).reshape(-1, 2)
    if len(curve) == 2:
        return linear_interpolation(curve[0], curve[1])
    diff = curve[1:] - curve[:-1]
    distance = np.linalg.norm(diff, axis=-1)
    norm_distance = distance / distance.sum()
    norm_distance = np.hstack(([0], norm_distance))
    cum_norm_dis = norm_distance.cumsum()
    pseudo_inv = np.linalg.pinv(bezier_coefficients(3, 4, cum_norm_dis))
    control_points = pseudo_inv.dot(curve)
    return control_points


def bezier2curve(bezier: np.ndarray, num_sample: int = 10):
    bezier = np.asarray(bezier)
    t = np.linspace(0, 1, num_sample)
    return np.array(bezier_coefficients(3, 4, t)).dot(bezier)


def poly2bezier(poly):
    poly = np.array(poly).reshape(-1, 2)
    points_num = len(poly)
    up_curve = poly[:points_num // 2]
    down_curve = poly[points_num // 2:]
    up_bezier = curve2bezier(up_curve)
    down_bezier = curve2bezier(down_curve)
    up_bezier[0] = up_curve[0]
    up_bezier[-1] = up_curve[-1]
    down_bezier[0] = down_curve[0]
    down_bezier[-1] = down_curve[-1]
    return np.vstack((up_bezier, down_bezier)).flatten().tolist()


def bezier2poly(bezier, num_sample=20):
    bezier = bezier.reshape(2, 4, 2)
    curve_top = bezier2curve(bezier[0], num_sample)
    curve_bottom = bezier2curve(bezier[1], num_sample)
    return np.vstack((curve_top, curve_bottom)).flatten().tolist()



# Copyright (c) OpenMMLab. All rights reserved.
from mmocr.registry import MODELS
from .encoder_decoder_recognizer import EncoderDecoderRecognizer


@MODELS.register_module()
class NRTR(EncoderDecoderRecognizer):
    """Implementation of `NRTR <https://arxiv.org/pdf/1806.00926.pdf>`_"""



# Copyright (c) OpenMMLab. All rights reserved.
import argparse
import os
import os.path as osp
import xml.etree.ElementTree as ET

import mmcv
import mmengine

from mmocr.utils import dump_ocr_data


def collect_files(data_dir):
    """Collect all images and their corresponding groundtruth files.

    Args:
        data_dir (str): The directory to dataset

    Returns:
        files (list): The list of tuples (img_file, groundtruth_file)
    """
    assert isinstance(data_dir, str)
    assert data_dir

    ann_list, imgs_list = [], []
    for video_dir in os.listdir(data_dir):
        for frame_dir in os.listdir(osp.join(data_dir, video_dir)):
            crt_dir = osp.join(data_dir, video_dir, frame_dir)
            if not osp.isdir(crt_dir):
                continue
            for crt_file in os.listdir(crt_dir):
                if crt_file.endswith('xml'):
                    ann_path = osp.join(crt_dir, crt_file)
                    img_path = osp.join(crt_dir,
                                        crt_file.replace('xml', 'png'))
                    if os.path.exists(img_path):
                        ann_list.append(ann_path)
                        imgs_list.append(img_path)
                    else:
                        continue

    files = list(zip(imgs_list, ann_list))
    assert len(files), f'No images found in {data_dir}'
    print(f'Loaded {len(files)} images from {data_dir}')

    return files


def collect_annotations(files, nproc=1):
    """Collect the annotation information.

    Args:
        files (list): The list of tuples (image_file, groundtruth_file)
        nproc (int): The number of process to collect annotations

    Returns:
        images (list): The list of image information dicts
    """
    assert isinstance(files, list)
    assert isinstance(nproc, int)

    if nproc > 1:
        images = mmengine.track_parallel_progress(
            load_img_info, files, nproc=nproc)
    else:
        images = mmengine.track_progress(load_img_info, files)

    return images


def load_img_info(files):
    """Load the information of one image.

    Args:
        files (tuple): The tuple of (img_file, groundtruth_file)

    Returns:
        img_info (dict): The dict of the img and annotation information
    """
    assert isinstance(files, tuple)

    img_file, gt_file = files
    assert osp.basename(gt_file).split('.')[0] == osp.basename(img_file).split(
        '.')[0]
    # read imgs while ignoring orientations
    img = mmcv.imread(img_file, 'unchanged')
    img_file = os.path.split(img_file)[-1]

    img_info = dict(
        file_name=img_file,
        height=img.shape[0],
        width=img.shape[1],
        segm_file=osp.join(osp.basename(gt_file)))

    if osp.splitext(gt_file)[1] == '.xml':
        img_info = load_xml_info(gt_file, img_info)
    else:
        raise NotImplementedError

    return img_info


def load_xml_info(gt_file, img_info):
    """Collect the annotation information.

    The annotation format is as the following:
    <annotation>
        <object>
            <name>hierarchy</name>
            <pose>Unspecified</pose>
            <truncated>0</truncated>
            <difficult>0</difficult>
            <bndbox>
                <xmin>657</xmin>
                <ymin>467</ymin>
                <xmax>839</xmax>
                <ymax>557</ymax>
            </bndbox>
        </object>
    </annotation>

    Args:
        gt_file (str): The path to ground-truth
        img_info (dict): The dict of the img and annotation information

    Returns:
        img_info (dict): The dict of the img and annotation information
    """

    obj = ET.parse(gt_file)
    root = obj.getroot()
    anno_info = []
    for obj in root.iter('object'):
        x = max(0, int(obj.find('bndbox').find('xmin').text))
        y = max(0, int(obj.find('bndbox').find('ymin').text))
        xmax = int(obj.find('bndbox').find('xmax').text)
        ymax = int(obj.find('bndbox').find('ymax').text)

        w, h = abs(xmax - x), abs(ymax - y)
        bbox = [x, y, w, h]
        segmentation = [x, y, x + w, y, x + w, y + h, x, y + h]

        anno = dict(
            iscrowd=0,
            category_id=1,
            bbox=bbox,
            area=w * h,
            segmentation=[segmentation])
        anno_info.append(anno)

    img_info.update(anno_info=anno_info)

    return img_info


def parse_args():
    parser = argparse.ArgumentParser(
        description='Generate training, val and test set of Lecture Video DB ')
    parser.add_argument('root_path', help='Root dir path of Lecture Video DB')
    parser.add_argument(
        '--nproc', default=1, type=int, help='number of process')
    args = parser.parse_args()
    return args


def main():
    args = parse_args()
    root_path = args.root_path

    for split in ['train', 'val', 'test']:
        print(f'Processing {split} set...')
        with mmengine.Timer(
                print_tmpl='It takes {}s to convert LV annotation'):
            files = collect_files(osp.join(root_path, 'imgs', split))
            image_infos = collect_annotations(files, nproc=args.nproc)
            dump_ocr_data(image_infos,
                          osp.join(root_path, 'instances_' + split + '.json'),
                          'textdet')


if __name__ == '__main__':
    main()



# Copyright (c) OpenMMLab. All rights reserved.
import torch
import torch.nn as nn
import torch.nn.functional as F


class DotProductAttentionLayer(nn.Module):

    def __init__(self, dim_model=None):
        super().__init__()

        self.scale = dim_model**-0.5 if dim_model is not None else 1.

    def forward(self, query, key, value, mask=None):
        n, seq_len = mask.size()
        logits = torch.matmul(query.permute(0, 2, 1), key) * self.scale

        if mask is not None:
            mask = mask.view(n, 1, seq_len)
            logits = logits.masked_fill(mask, float('-inf'))

        weights = F.softmax(logits, dim=2)

        glimpse = torch.matmul(weights, value.transpose(1, 2))

        glimpse = glimpse.permute(0, 2, 1).contiguous()

        return glimpse



# Copyright (c) OpenMMLab. All rights reserved.
import argparse
import os
import os.path as osp
import shutil
import xml.etree.ElementTree as ET
import zipfile
from xml.etree.ElementTree import ParseError


def extract(root_path):
    idx = 0
    for language in ['English', 'Korean', 'Mixed']:
        for camera in ['Digital_Camera', 'Mobile_Phone']:
            crt_path = osp.join(root_path, 'KAIST', language, camera)
            zips = os.listdir(crt_path)
            for zip in zips:
                extracted_path = osp.join(root_path, 'tmp', zip)
                extract_zipfile(osp.join(crt_path, zip), extracted_path)
                for file in os.listdir(extracted_path):
                    if file.endswith('xml'):
                        src_ann = os.path.join(extracted_path, file)
                        # Filtering broken annotations
                        try:
                            ET.parse(src_ann)
                        except ParseError:
                            continue
                        src_img = None
                        img_names = [
                            file.replace('xml', suffix)
                            for suffix in ['jpg', 'JPG']
                        ]
                        for im in img_names:
                            img_path = osp.join(extracted_path, im)
                            if osp.exists(img_path):
                                src_img = img_path
                        if src_img:
                            shutil.move(
                                src_ann,
                                osp.join(root_path, 'annotations',
                                         str(idx).zfill(5) + '.xml'))
                            shutil.move(
                                src_img,
                                osp.join(root_path, 'imgs',
                                         str(idx).zfill(5) + '.jpg'))
                            idx += 1


def extract_zipfile(zip_path, dst_dir, delete=True):

    files = zipfile.ZipFile(zip_path)
    for file in files.namelist():
        files.extract(file, dst_dir)
    if delete:
        os.remove(zip_path)


def parse_args():
    parser = argparse.ArgumentParser(description='Extract KAIST zips')
    parser.add_argument('root_path', help='Root path of KAIST')
    args = parser.parse_args()
    return args


def main():
    args = parse_args()
    root_path = args.root_path
    assert osp.exists(root_path)
    extract(root_path)
    shutil.rmtree(osp.join(args.root_path, 'tmp'))
    shutil.rmtree(osp.join(args.root_path, 'KAIST'))


if __name__ == '__main__':
    main()



# Copyright (c) OpenMMLab. All rights reserved.

from typing import Dict, Sequence, Union

import torch
import torch.nn as nn

from mmocr.models.common.dictionary import Dictionary
from mmocr.registry import TASK_UTILS
from mmocr.structures import TextRecogDataSample


class BaseTextRecogModuleLoss(nn.Module):
    """Base recognition loss.

    Args:
        dictionary (dict or :obj:`Dictionary`): The config for `Dictionary` or
            the instance of `Dictionary`.
        max_seq_len (int): Maximum sequence length. The sequence is usually
            generated from decoder. Defaults to 40.
        letter_case (str): There are three options to alter the letter cases
            of gt texts:
            - unchanged: Do not change gt texts.
            - upper: Convert gt texts into uppercase characters.
            - lower: Convert gt texts into lowercase characters.
            Usually, it only works for English characters. Defaults to
            'unchanged'.
        pad_with (str): The padding strategy for ``gt_text.padded_indexes``.
            Defaults to 'auto'. Options are:
            - 'auto': Use dictionary.padding_idx to pad gt texts, or
              dictionary.end_idx if dictionary.padding_idx
              is None.
            - 'padding': Always use dictionary.padding_idx to pad gt texts.
            - 'end': Always use dictionary.end_idx to pad gt texts.
            - 'none': Do not pad gt texts.
    """

    def __init__(self,
                 dictionary: Union[Dict, Dictionary],
                 max_seq_len: int = 40,
                 letter_case: str = 'unchanged',
                 pad_with: str = 'auto',
                 **kwargs) -> None:
        super().__init__()
        if isinstance(dictionary, dict):
            self.dictionary = TASK_UTILS.build(dictionary)
        elif isinstance(dictionary, Dictionary):
            self.dictionary = dictionary
        else:
            raise TypeError(
                'The type of dictionary should be `Dictionary` or dict, '
                f'but got {type(dictionary)}')
        self.max_seq_len = max_seq_len
        assert letter_case in ['unchanged', 'upper', 'lower']
        self.letter_case = letter_case

        assert pad_with in ['auto', 'padding', 'end', 'none']
        if pad_with == 'auto':
            self.pad_idx = self.dictionary.padding_idx or \
                self.dictionary.end_idx
        elif pad_with == 'padding':
            self.pad_idx = self.dictionary.padding_idx
        elif pad_with == 'end':
            self.pad_idx = self.dictionary.end_idx
        else:
            self.pad_idx = None
        if self.pad_idx is None and pad_with != 'none':
            if pad_with == 'auto':
                raise ValueError('pad_with="auto", but dictionary.end_idx'
                                 ' and dictionary.padding_idx are both None')
            else:
                raise ValueError(
                    f'pad_with="{pad_with}", but dictionary.{pad_with}_idx is'
                    ' None')

    def get_targets(
        self, data_samples: Sequence[TextRecogDataSample]
    ) -> Sequence[TextRecogDataSample]:
        """Target generator.

        Args:
            data_samples (list[TextRecogDataSample]): It usually includes
                ``gt_text`` information.

        Returns:
            list[TextRecogDataSample]: Updated data_samples. Two keys will be
            added to data_sample:

            - indexes (torch.LongTensor): Character indexes representing gt
              texts. All special tokens are excluded, except for UKN.
            - padded_indexes (torch.LongTensor): Character indexes
              representing gt texts with BOS and EOS if applicable, following
              several padding indexes until the length reaches ``max_seq_len``.
              In particular, if ``pad_with='none'``, no padding will be
              applied.
        """

        for data_sample in data_samples:
            if data_sample.get('have_target', False):
                continue
            text = data_sample.gt_text.item
            if self.letter_case in ['upper', 'lower']:
                text = getattr(text, self.letter_case)()
            indexes = self.dictionary.str2idx(text)
            indexes = torch.LongTensor(indexes)

            # target indexes for loss
            src_target = torch.LongTensor(indexes.size(0) + 2).fill_(0)
            src_target[1:-1] = indexes
            if self.dictionary.start_idx is not None:
                src_target[0] = self.dictionary.start_idx
                slice_start = 0
            else:
                slice_start = 1
            if self.dictionary.end_idx is not None:
                src_target[-1] = self.dictionary.end_idx
                slice_end = src_target.size(0)
            else:
                slice_end = src_target.size(0) - 1
            src_target = src_target[slice_start:slice_end]
            if self.pad_idx is not None:
                padded_indexes = (torch.ones(self.max_seq_len) *
                                  self.pad_idx).long()
                char_num = min(src_target.size(0), self.max_seq_len)
                padded_indexes[:char_num] = src_target[:char_num]
            else:
                padded_indexes = src_target
            # put in DataSample
            data_sample.gt_text.indexes = indexes
            data_sample.gt_text.padded_indexes = padded_indexes
            data_sample.set_metainfo(dict(have_target=True))
        return data_samples



_base_ = [
    '../_base_/datasets/toy_data.py',
    '../_base_/default_runtime.py',
    '../_base_/schedules/schedule_adam_step_5e.py',
    '_base_robustscanner_resnet31.py',
]

# dataset settings
train_list = [_base_.toy_rec_train]
test_list = [_base_.toy_rec_test]

default_hooks = dict(logger=dict(type='LoggerHook', interval=100))

train_dataloader = dict(
    batch_size=64,
    num_workers=8,
    persistent_workers=True,
    sampler=dict(type='DefaultSampler', shuffle=True),
    dataset=dict(
        type='ConcatDataset',
        datasets=train_list,
        pipeline=_base_.train_pipeline))

val_dataloader = dict(
    batch_size=1,
    num_workers=4,
    persistent_workers=True,
    drop_last=False,
    sampler=dict(type='DefaultSampler', shuffle=False),
    dataset=dict(
        type='ConcatDataset',
        datasets=test_list,
        pipeline=_base_.test_pipeline))
test_dataloader = val_dataloader

val_evaluator = dict(dataset_prefixes=['Toy'])
test_evaluator = val_evaluator



# Copyright (c) OpenMMLab. All rights reserved.
from .dataset_wrapper import ConcatDataset
from .icdar_dataset import IcdarDataset
from .ocr_dataset import OCRDataset
from .recog_lmdb_dataset import RecogLMDBDataset
from .recog_text_dataset import RecogTextDataset
from .samplers import *  # NOQA
from .transforms import *  # NOQA
from .wildreceipt_dataset import WildReceiptDataset

__all__ = [
    'IcdarDataset', 'OCRDataset', 'RecogLMDBDataset', 'RecogTextDataset',
    'WildReceiptDataset', 'ConcatDataset'
]



_base_ = [
    '_base_drrg_resnet50_fpn-unet.py',
    '../_base_/datasets/ctw1500.py',
    '../_base_/default_runtime.py',
    '../_base_/schedules/schedule_sgd_1200e.py',
]

# dataset settings
ctw1500_textdet_train = _base_.ctw1500_textdet_train
ctw1500_textdet_train.pipeline = _base_.train_pipeline
ctw1500_textdet_test = _base_.ctw1500_textdet_test
ctw1500_textdet_test.pipeline = _base_.test_pipeline

train_dataloader = dict(
    batch_size=4,
    num_workers=4,
    persistent_workers=True,
    sampler=dict(type='DefaultSampler', shuffle=True),
    dataset=ctw1500_textdet_train)

val_dataloader = dict(
    batch_size=1,
    num_workers=1,
    persistent_workers=True,
    sampler=dict(type='DefaultSampler', shuffle=False),
    dataset=ctw1500_textdet_test)

test_dataloader = val_dataloader

auto_scale_lr = dict(base_batch_size=16)



dictionary = dict(
    type='Dictionary',
    dict_file='{{ fileDirname }}/../../../dicts/english_digits_symbols.txt',
    with_padding=True,
    with_unknown=True,
    same_start_end=True,
    with_start=True,
    with_end=True)

model = dict(
    type='NRTR',
    backbone=dict(type='NRTRModalityTransform'),
    encoder=dict(type='NRTREncoder', n_layers=12),
    decoder=dict(
        type='NRTRDecoder',
        module_loss=dict(
            type='CEModuleLoss', ignore_first_char=True, flatten=True),
        postprocessor=dict(type='AttentionPostprocessor'),
        dictionary=dictionary,
        max_seq_len=30),
    data_preprocessor=dict(
        type='TextRecogDataPreprocessor',
        mean=[123.675, 116.28, 103.53],
        std=[58.395, 57.12, 57.375]))

train_pipeline = [
    dict(type='LoadImageFromFile', ignore_empty=True, min_size=2),
    dict(type='LoadOCRAnnotations', with_text=True),
    dict(
        type='RescaleToHeight',
        height=32,
        min_width=32,
        max_width=160,
        width_divisor=4),
    dict(type='PadToWidth', width=160),
    dict(
        type='PackTextRecogInputs',
        meta_keys=('img_path', 'ori_shape', 'img_shape', 'valid_ratio'))
]

test_pipeline = [
    dict(type='LoadImageFromFile'),
    dict(
        type='RescaleToHeight',
        height=32,
        min_width=32,
        max_width=160,
        width_divisor=16),
    dict(type='PadToWidth', width=160),
    # add loading annotation after ``Resize`` because ground truth
    # does not need to do resize data transform
    dict(type='LoadOCRAnnotations', with_text=True),
    dict(
        type='PackTextRecogInputs',
        meta_keys=('img_path', 'ori_shape', 'img_shape', 'valid_ratio'))
]

tta_pipeline = [
    dict(type='LoadImageFromFile'),
    dict(
        type='TestTimeAug',
        transforms=[
            [
                dict(
                    type='ConditionApply',
                    true_transforms=[
                        dict(
                            type='ImgAugWrapper',
                            args=[dict(cls='Rot90', k=0, keep_size=False)])
                    ],
                    condition="results['img_shape'][1]<results['img_shape'][0]"
                ),
                dict(
                    type='ConditionApply',
                    true_transforms=[
                        dict(
                            type='ImgAugWrapper',
                            args=[dict(cls='Rot90', k=1, keep_size=False)])
                    ],
                    condition="results['img_shape'][1]<results['img_shape'][0]"
                ),
                dict(
                    type='ConditionApply',
                    true_transforms=[
                        dict(
                            type='ImgAugWrapper',
                            args=[dict(cls='Rot90', k=3, keep_size=False)])
                    ],
                    condition="results['img_shape'][1]<results['img_shape'][0]"
                ),
            ],
            [
                dict(
                    type='RescaleToHeight',
                    height=32,
                    min_width=32,
                    max_width=160,
                    width_divisor=16)
            ],
            [dict(type='PadToWidth', width=160)],
            # add loading annotation after ``Resize`` because ground truth
            # does not need to do resize data transform
            [dict(type='LoadOCRAnnotations', with_text=True)],
            [
                dict(
                    type='PackTextRecogInputs',
                    meta_keys=('img_path', 'ori_shape', 'img_shape',
                               'valid_ratio'))
            ]
        ])
]



# Copyright (c) OpenMMLab. All rights reserved.
import argparse
import math
import os
import os.path as osp

import cv2
import mmcv
import mmengine
from PIL import Image

from mmocr.utils import crop_img, dump_ocr_data


def collect_files(img_dir, gt_dir, ratio):
    """Collect all images and their corresponding groundtruth files.
    Args:
        img_dir (str): The image directory
        gt_dir (str): The groundtruth directory
        ratio (float): Split ratio for val set

    Returns:
        files (list): The list of tuples (img_file, groundtruth_file)
    """
    assert isinstance(img_dir, str)
    assert img_dir
    assert isinstance(gt_dir, str)
    assert gt_dir
    assert isinstance(ratio, float)
    assert ratio < 1.0, 'val_ratio should be a float between 0.0 to 1.0'

    ann_list, imgs_list = [], []
    for ann_file in os.listdir(gt_dir):
        img_file = osp.join(img_dir, ann_file.replace('txt', 'jpg'))
        # This dataset contains some images obtained from .gif,
        # which cannot be loaded by mmcv.imread(), convert them
        # to RGB mode.
        try:
            if mmcv.imread(img_file) is None:
                print(f'Convert {img_file} to RGB mode.')
                img = Image.open(img_file)
                img = img.convert('RGB')
                img.save(img_file)
        except cv2.error:
            print(f'Skip broken img {img_file}')
            continue

        ann_list.append(osp.join(gt_dir, ann_file))
        imgs_list.append(img_file)

    all_files = list(zip(imgs_list, ann_list))
    assert len(all_files), f'No images found in {img_dir}'
    print(f'Loaded {len(all_files)} images from {img_dir}')

    trn_files, val_files = [], []
    if ratio > 0:
        for i, file in enumerate(all_files):
            if i % math.floor(1 / ratio):
                trn_files.append(file)
            else:
                val_files.append(file)
    else:
        trn_files, val_files = all_files, []

    print(f'training #{len(trn_files)}, val #{len(val_files)}')

    return trn_files, val_files


def collect_annotations(files, nproc=1):
    """Collect the annotation information.
    Args:
        files (list): The list of tuples (image_file, groundtruth_file)
        nproc (int): The number of process to collect annotations

    Returns:
        images (list): The list of image information dicts
    """
    assert isinstance(files, list)
    assert isinstance(nproc, int)

    if nproc > 1:
        images = mmengine.track_parallel_progress(
            load_img_info, files, nproc=nproc)
    else:
        images = mmengine.track_progress(load_img_info, files)

    return images


def load_img_info(files):
    """Load the information of one image.
    Args:
        files (tuple): The tuple of (img_file, groundtruth_file)

    Returns:
        img_info (dict): The dict of the img and annotation information
    """
    assert isinstance(files, tuple)

    img_file, gt_file = files
    assert osp.basename(gt_file).split('.')[0] == osp.basename(img_file).split(
        '.')[0]
    # read imgs while ignoring orientations
    img = mmcv.imread(img_file)

    img_info = dict(
        file_name=osp.join(osp.basename(img_file)),
        height=img.shape[0],
        width=img.shape[1],
        segm_file=osp.join(osp.basename(gt_file)))

    if osp.splitext(gt_file)[1] == '.txt':
        img_info = load_txt_info(gt_file, img_info)
    else:
        raise NotImplementedError

    return img_info


def load_txt_info(gt_file, img_info):
    """Collect the annotation information.

    The annotation format is as the following:
    x1,y1,x2,y2,x3,y3,x4,y4,text

    45.45,226.83,11.87,181.79,183.84,13.1,233.79,49.95,时尚袋袋
    345.98,311.18,345.98,347.21,462.26,347.21,462.26,311.18,73774
    462.26,292.34,461.44,299.71,502.39,299.71,502.39,292.34,73/74/737

    Args:
        gt_file (str): The path to ground-truth
        img_info (dict): The dict of the img and annotation information

    Returns:
        img_info (dict): The dict of the img and annotation information
    """

    anno_info = []
    with open(gt_file) as f:
        lines = f.readlines()
    for line in lines:
        points = line.split(',')[0:8]
        word = line.split(',')[8].rstrip('\n')
        if word == '###':
            continue
        bbox = [math.floor(float(pt)) for pt in points]
        anno = dict(bbox=bbox, word=word)
        anno_info.append(anno)

    img_info.update(anno_info=anno_info)

    return img_info


def generate_ann(root_path, split, image_infos, preserve_vertical):
    """Generate cropped annotations and label txt file.

    Args:
        root_path (str): The root path of the dataset
        split (str): The split of dataset. Namely: training or test
        image_infos (list[dict]): A list of dicts of the img and
            annotation information
        preserve_vertical (bool): Whether to preserve vertical texts
    """
    print('Cropping images...')
    dst_image_root = osp.join(root_path, 'crops', split)
    ignore_image_root = osp.join(root_path, 'ignores', split)
    if split == 'training':
        dst_label_file = osp.join(root_path, 'train_label.json')
    elif split == 'val':
        dst_label_file = osp.join(root_path, 'val_label.json')
    mmengine.mkdir_or_exist(dst_image_root)
    mmengine.mkdir_or_exist(ignore_image_root)

    img_info = []
    for image_info in image_infos:
        index = 1
        src_img_path = osp.join(root_path, 'imgs', image_info['file_name'])
        image = mmcv.imread(src_img_path)
        src_img_root = image_info['file_name'].split('.')[0]

        for anno in image_info['anno_info']:
            word = anno['word']
            dst_img = crop_img(image, anno['bbox'], 0, 0)
            h, w, _ = dst_img.shape

            dst_img_name = f'{src_img_root}_{index}.png'
            index += 1
            # Skip invalid annotations
            if min(dst_img.shape) == 0:
                continue
            # Skip vertical texts
            if not preserve_vertical and h / w > 2 and split == 'training':
                dst_img_path = osp.join(ignore_image_root, dst_img_name)
                mmcv.imwrite(dst_img, dst_img_path)
                continue

            dst_img_path = osp.join(dst_image_root, dst_img_name)
            mmcv.imwrite(dst_img, dst_img_path)

            img_info.append({
                'file_name': dst_img_name,
                'anno_info': [{
                    'text': word
                }]
            })

    dump_ocr_data(img_info, dst_label_file, 'textrecog')


def parse_args():
    parser = argparse.ArgumentParser(
        description='Generate training and val set of MTWI.')
    parser.add_argument('root_path', help='Root dir path of MTWI')
    parser.add_argument(
        '--val-ratio', help='Split ratio for val set', default=0.0, type=float)
    parser.add_argument(
        '--preserve-vertical',
        help='Preserve samples containing vertical texts',
        action='store_true')
    parser.add_argument(
        '--nproc', default=1, type=int, help='Number of process')
    args = parser.parse_args()
    return args


def main():
    args = parse_args()
    root_path = args.root_path
    ratio = args.val_ratio

    trn_files, val_files = collect_files(
        osp.join(root_path, 'imgs'), osp.join(root_path, 'annotations'), ratio)

    # Train set
    trn_infos = collect_annotations(trn_files, nproc=args.nproc)
    with mmengine.Timer(
            print_tmpl='It takes {}s to convert MTWI Training annotation'):
        generate_ann(root_path, 'training', trn_infos, args.preserve_vertical)

    # Val set
    if len(val_files) > 0:
        val_infos = collect_annotations(val_files, nproc=args.nproc)
        with mmengine.Timer(
                print_tmpl='It takes {}s to convert MTWI Val annotation'):
            generate_ann(root_path, 'val', val_infos, args.preserve_vertical)


if __name__ == '__main__':
    main()



# Copyright (c) OpenMMLab. All rights reserved.
import datetime
import warnings

from mmengine.registry import DefaultScope


def register_all_modules(init_default_scope: bool = True) -> None:
    """Register all modules in mmocr into the registries.

    Args:
        init_default_scope (bool): Whether initialize the mmocr default scope.
            When `init_default_scope=True`, the global default scope will be
            set to `mmocr`, and all registries will build modules from mmocr's
            registry node. To understand more about the registry, please refer
            to https://github.com/open-mmlab/mmengine/blob/main/docs/en/tutorials/registry.md
            Defaults to True.
    """  # noqa
    import mmocr.apis  # noqa: F401,F403
    import mmocr.datasets  # noqa: F401,F403
    import mmocr.engine  # noqa: F401,F403
    import mmocr.evaluation  # noqa: F401,F403
    import mmocr.models  # noqa: F401,F403
    import mmocr.structures  # noqa: F401,F403
    import mmocr.visualization  # noqa: F401,F403
    if init_default_scope:
        never_created = DefaultScope.get_current_instance() is None \
                        or not DefaultScope.check_instance_created('mmocr')
        if never_created:
            DefaultScope.get_instance('mmocr', scope_name='mmocr')
            return
        current_scope = DefaultScope.get_current_instance()
        if current_scope.scope_name != 'mmocr':
            warnings.warn('The current default scope '
                          f'"{current_scope.scope_name}" is not "mmocr", '
                          '`register_all_modules` will force the current'
                          'default scope to be "mmocr". If this is not '
                          'expected, please set `init_default_scope=False`.')
            # avoid name conflict
            new_instance_name = f'mmocr-{datetime.datetime.now()}'
            DefaultScope.get_instance(new_instance_name, scope_name='mmocr')



svt_textrecog_data_root = 'data/svt'

svt_textrecog_train = dict(
    type='OCRDataset',
    data_root=svt_textrecog_data_root,
    ann_file='textrecog_train.json',
    pipeline=None)

svt_textrecog_test = dict(
    type='OCRDataset',
    data_root=svt_textrecog_data_root,
    ann_file='textrecog_test.json',
    test_mode=True,
    pipeline=None)



# Copyright (c) OpenMMLab. All rights reserved.
from .mini_vgg import MiniVGG
from .mobilenet_v2 import MobileNetV2
from .nrtr_modality_transformer import NRTRModalityTransform
from .resnet import ResNet
from .resnet31_ocr import ResNet31OCR
from .resnet_abi import ResNetABI
from .shallow_cnn import ShallowCNN

__all__ = [
    'ResNet31OCR', 'MiniVGG', 'NRTRModalityTransform', 'ShallowCNN',
    'ResNetABI', 'ResNet', 'MobileNetV2'
]



# Copyright (c) OpenMMLab. All rights reserved.
from typing import Optional, Union

import torch
import torch.nn as nn

from mmocr.registry import MODELS


@MODELS.register_module()
class MaskedBalancedBCEWithLogitsLoss(nn.Module):
    """This loss combines a Sigmoid layers and a masked balanced BCE loss in
    one single class. It's AMP-eligible.

    Args:
        reduction (str, optional): The method to reduce the loss.
            Options are 'none', 'mean' and 'sum'. Defaults to 'none'.
        negative_ratio (float or int, optional): Maximum ratio of negative
            samples to positive ones. Defaults to 3.
        fallback_negative_num (int, optional): When the mask contains no
            positive samples, the number of negative samples to be sampled.
            Defaults to 0.
        eps (float, optional): Eps to avoid zero-division error.  Defaults to
            1e-6.
    """

    def __init__(self,
                 reduction: str = 'none',
                 negative_ratio: Union[float, int] = 3,
                 fallback_negative_num: int = 0,
                 eps: float = 1e-6) -> None:
        super().__init__()
        assert reduction in ['none', 'mean', 'sum']
        assert isinstance(negative_ratio, (float, int))
        assert isinstance(fallback_negative_num, int)
        assert isinstance(eps, float)
        self.eps = eps
        self.negative_ratio = negative_ratio
        self.reduction = reduction
        self.fallback_negative_num = fallback_negative_num
        self.loss = nn.BCEWithLogitsLoss(reduction=reduction)

    def forward(self,
                pred: torch.Tensor,
                gt: torch.Tensor,
                mask: Optional[torch.Tensor] = None) -> torch.Tensor:
        """Forward function.

        Args:
            pred (torch.Tensor): The prediction in any shape.
            gt (torch.Tensor): The learning target of the prediction in the
                same shape as pred.
            mask (torch.Tensor, optional): Binary mask in the same shape of
                pred, indicating positive regions to calculate the loss. Whole
                region will be taken into account if not provided. Defaults to
                None.

        Returns:
            torch.Tensor: The loss value.
        """

        assert pred.size() == gt.size() and gt.numel() > 0
        if mask is None:
            mask = torch.ones_like(gt)
        assert mask.size() == gt.size()

        positive = (gt * mask).float()
        negative = ((1 - gt) * mask).float()
        positive_count = int(positive.sum())
        if positive_count == 0:
            negative_count = min(
                int(negative.sum()), self.fallback_negative_num)
        else:
            negative_count = min(
                int(negative.sum()), int(positive_count * self.negative_ratio))

        assert gt.max() <= 1 and gt.min() >= 0
        loss = self.loss(pred, gt)
        positive_loss = loss * positive
        negative_loss = loss * negative

        negative_loss, _ = torch.topk(negative_loss.view(-1), negative_count)

        balance_loss = (positive_loss.sum() + negative_loss.sum()) / (
            positive_count + negative_count + self.eps)

        return balance_loss


@MODELS.register_module()
class MaskedBalancedBCELoss(MaskedBalancedBCEWithLogitsLoss):
    """Masked Balanced BCE loss.

    Args:
        reduction (str, optional): The method to reduce the loss.
            Options are 'none', 'mean' and 'sum'. Defaults to 'none'.
        negative_ratio (float or int): Maximum ratio of negative
            samples to positive ones. Defaults to 3.
        fallback_negative_num (int): When the mask contains no
            positive samples, the number of negative samples to be sampled.
            Defaults to 0.
        eps (float): Eps to avoid zero-division error.  Defaults to
            1e-6.
    """

    def __init__(self,
                 reduction: str = 'none',
                 negative_ratio: Union[float, int] = 3,
                 fallback_negative_num: int = 0,
                 eps: float = 1e-6) -> None:
        super().__init__()
        assert reduction in ['none', 'mean', 'sum']
        assert isinstance(negative_ratio, (float, int))
        assert isinstance(fallback_negative_num, int)
        assert isinstance(eps, float)
        self.eps = eps
        self.negative_ratio = negative_ratio
        self.reduction = reduction
        self.fallback_negative_num = fallback_negative_num
        self.loss = nn.BCELoss(reduction=reduction)

    def forward(self,
                pred: torch.Tensor,
                gt: torch.Tensor,
                mask: Optional[torch.Tensor] = None) -> torch.Tensor:
        """Forward function.

        Args:
            pred (torch.Tensor): The prediction in any shape.
            gt (torch.Tensor): The learning target of the prediction in the
                same shape as pred.
            mask (torch.Tensor, optional): Binary mask in the same shape of
                pred, indicating positive regions to calculate the loss. Whole
                region will be taken into account if not provided. Defaults to
                None.

        Returns:
            torch.Tensor: The loss value.
        """

        assert pred.max() <= 1 and pred.min() >= 0
        return super().forward(pred, gt, mask)


@MODELS.register_module()
class MaskedBCEWithLogitsLoss(nn.Module):
    """This loss combines a Sigmoid layers and a masked BCE loss in one single
    class. It's AMP-eligible.

    Args:
        eps (float): Eps to avoid zero-division error.  Defaults to
            1e-6.
    """

    def __init__(self, eps: float = 1e-6) -> None:
        super().__init__()
        assert isinstance(eps, float)
        self.eps = eps
        self.loss = nn.BCEWithLogitsLoss(reduction='none')

    def forward(self,
                pred: torch.Tensor,
                gt: torch.Tensor,
                mask: Optional[torch.Tensor] = None) -> torch.Tensor:
        """Forward function.

        Args:
            pred (torch.Tensor): The prediction in any shape.
            gt (torch.Tensor): The learning target of the prediction in the
                same shape as pred.
            mask (torch.Tensor, optional): Binary mask in the same shape of
                pred, indicating positive regions to calculate the loss. Whole
                region will be taken into account if not provided. Defaults to
                None.

        Returns:
            torch.Tensor: The loss value.
        """

        assert pred.size() == gt.size() and gt.numel() > 0
        if mask is None:
            mask = torch.ones_like(gt)
        assert mask.size() == gt.size()

        assert gt.max() <= 1 and gt.min() >= 0
        loss = self.loss(pred, gt)

        return (loss * mask).sum() / (mask.sum() + self.eps)


@MODELS.register_module()
class MaskedBCELoss(MaskedBCEWithLogitsLoss):
    """Masked BCE loss.

    Args:
        eps (float): Eps to avoid zero-division error.  Defaults to
            1e-6.
    """

    def __init__(self, eps: float = 1e-6) -> None:
        super().__init__()
        assert isinstance(eps, float)
        self.eps = eps
        self.loss = nn.BCELoss(reduction='none')

    def forward(self,
                pred: torch.Tensor,
                gt: torch.Tensor,
                mask: Optional[torch.Tensor] = None) -> torch.Tensor:
        """Forward function.

        Args:
            pred (torch.Tensor): The prediction in any shape.
            gt (torch.Tensor): The learning target of the prediction in the
                same shape as pred.
            mask (torch.Tensor, optional): Binary mask in the same shape of
                pred, indicating positive regions to calculate the loss. Whole
                region will be taken into account if not provided. Defaults to
                None.

        Returns:
            torch.Tensor: The loss value.
        """

        assert pred.max() <= 1 and pred.min() >= 0

        return super().forward(pred, gt, mask)



# Copyright (c) OpenMMLab. All rights reserved.
from typing import Optional, Sequence, Tuple

import torch

from mmocr.registry import MODELS
from mmocr.structures import TextRecogDataSample
from .base import BaseTextRecogPostprocessor


@MODELS.register_module()
class AttentionPostprocessor(BaseTextRecogPostprocessor):
    """PostProcessor for seq2seq."""

    def get_single_prediction(
        self,
        probs: torch.Tensor,
        data_sample: Optional[TextRecogDataSample] = None,
    ) -> Tuple[Sequence[int], Sequence[float]]:
        """Convert the output probabilities of a single image to index and
        score.

        Args:
            probs (torch.Tensor): Character probabilities with shape
                :math:`(T, C)`.
            data_sample (TextRecogDataSample, optional): Datasample of an
                image. Defaults to None.

        Returns:
            tuple(list[int], list[float]): index and score.
        """
        max_value, max_idx = torch.max(probs, -1)
        index, score = [], []
        output_index = max_idx.cpu().detach().numpy().tolist()
        output_score = max_value.cpu().detach().numpy().tolist()
        for char_index, char_score in zip(output_index, output_score):
            if char_index in self.ignore_indexes:
                continue
            if char_index == self.dictionary.end_idx:
                break
            index.append(char_index)
            score.append(char_score)
        return index, score



# Copyright (c) OpenMMLab. All rights reserved.
from typing import Dict, List, Optional, Sequence, Tuple, Union

import torch
import torch.nn as nn
from mmcv.cnn import ConvModule

from mmocr.models.common.dictionary import Dictionary
from mmocr.models.common.modules import PositionalEncoding
from mmocr.registry import MODELS
from mmocr.structures import TextRecogDataSample
from .base import BaseDecoder


@MODELS.register_module()
class ABIVisionDecoder(BaseDecoder):
    """Converts visual features into text characters.

    Implementation of VisionEncoder in
        `ABINet <https://arxiv.org/abs/2103.06495>`_.

    Args:
        dictionary (dict or :obj:`Dictionary`): The config for `Dictionary` or
            the instance of `Dictionary`.
        in_channels (int): Number of channels :math:`E` of input vector.
            Defaults to 512.
        num_channels (int): Number of channels of hidden vectors in mini U-Net.
            Defaults to 64.
        attn_height (int): Height :math:`H` of input image features. Defaults
            to 8.
        attn_width (int): Width :math:`W` of input image features. Defaults to
            32.
        attn_mode (str): Upsampling mode for :obj:`torch.nn.Upsample` in mini
            U-Net. Defaults to 'nearest'.
        module_loss (dict, optional): Config to build loss. Defaults to None.
        postprocessor (dict, optional): Config to build postprocessor.
            Defaults to None.
        max_seq_len (int): Maximum sequence length. The
            sequence is usually generated from decoder. Defaults to 40.
        init_cfg (dict or list[dict], optional): Initialization configs.
            Defaults to dict(type='Xavier', layer='Conv2d').
    """

    def __init__(self,
                 dictionary: Union[Dict, Dictionary],
                 in_channels: int = 512,
                 num_channels: int = 64,
                 attn_height: int = 8,
                 attn_width: int = 32,
                 attn_mode: str = 'nearest',
                 module_loss: Optional[Dict] = None,
                 postprocessor: Optional[Dict] = None,
                 max_seq_len: int = 40,
                 init_cfg: Optional[Union[Dict, List[Dict]]] = dict(
                     type='Xavier', layer='Conv2d'),
                 **kwargs) -> None:

        super().__init__(
            dictionary=dictionary,
            module_loss=module_loss,
            postprocessor=postprocessor,
            max_seq_len=max_seq_len,
            init_cfg=init_cfg)

        # For mini-Unet
        self.k_encoder = nn.Sequential(
            self._encoder_layer(in_channels, num_channels, stride=(1, 2)),
            self._encoder_layer(num_channels, num_channels, stride=(2, 2)),
            self._encoder_layer(num_channels, num_channels, stride=(2, 2)),
            self._encoder_layer(num_channels, num_channels, stride=(2, 2)))

        self.k_decoder = nn.Sequential(
            self._decoder_layer(
                num_channels, num_channels, scale_factor=2, mode=attn_mode),
            self._decoder_layer(
                num_channels, num_channels, scale_factor=2, mode=attn_mode),
            self._decoder_layer(
                num_channels, num_channels, scale_factor=2, mode=attn_mode),
            self._decoder_layer(
                num_channels,
                in_channels,
                size=(attn_height, attn_width),
                mode=attn_mode))

        self.pos_encoder = PositionalEncoding(in_channels, max_seq_len)
        self.project = nn.Linear(in_channels, in_channels)
        self.cls = nn.Linear(in_channels, self.dictionary.num_classes)

    def forward_train(
            self,
            feat: Optional[torch.Tensor] = None,
            out_enc: torch.Tensor = None,
            data_samples: Optional[Sequence[TextRecogDataSample]] = None
    ) -> Dict:
        """
        Args:
            feat (Tensor, optional): Image features of shape (N, E, H, W).
                Defaults to None.
            out_enc (torch.Tensor): Encoder output. Defaults to None.
            data_samples (list[TextRecogDataSample], optional): Batch of
                TextRecogDataSample, containing gt_text information. Defaults
                to None.

        Returns:
            dict: A dict with keys ``feature``, ``logits`` and ``attn_scores``.

            - feature (Tensor): Shape (N, T, E). Raw visual features for
              language decoder.
            - logits (Tensor): Shape (N, T, C). The raw logits for
              characters.
            - attn_scores (Tensor): Shape (N, T, H, W). Intermediate result
              for vision-language aligner.
        """
        # Position Attention
        N, E, H, W = out_enc.size()
        k, v = out_enc, out_enc  # (N, E, H, W)

        # Apply mini U-Net on k
        features = []
        for i in range(len(self.k_encoder)):
            k = self.k_encoder[i](k)
            features.append(k)
        for i in range(len(self.k_decoder) - 1):
            k = self.k_decoder[i](k)
            k = k + features[len(self.k_decoder) - 2 - i]
        k = self.k_decoder[-1](k)

        # q = positional encoding
        zeros = out_enc.new_zeros((N, self.max_seq_len, E))  # (N, T, E)
        q = self.pos_encoder(zeros)  # (N, T, E)
        q = self.project(q)  # (N, T, E)

        # Attention encoding
        attn_scores = torch.bmm(q, k.flatten(2, 3))  # (N, T, (H*W))
        attn_scores = attn_scores / (E**0.5)
        attn_scores = torch.softmax(attn_scores, dim=-1)
        v = v.permute(0, 2, 3, 1).view(N, -1, E)  # (N, (H*W), E)
        attn_vecs = torch.bmm(attn_scores, v)  # (N, T, E)

        out_enc = self.cls(attn_vecs)
        result = {
            'feature': attn_vecs,
            'logits': out_enc,
            'attn_scores': attn_scores.view(N, -1, H, W)
        }
        return result

    def forward_test(
            self,
            feat: Optional[torch.Tensor] = None,
            out_enc: torch.Tensor = None,
            data_samples: Optional[Sequence[TextRecogDataSample]] = None
    ) -> Dict:
        """
        Args:
            feat (torch.Tensor, optional): Image features of shape
                (N, E, H, W). Defaults to None.
            out_enc (torch.Tensor): Encoder output. Defaults to None.
            data_samples (list[TextRecogDataSample], optional): Batch of
                TextRecogDataSample, containing gt_text information. Defaults
                to None.

        Returns:
            dict: A dict with keys ``feature``, ``logits`` and ``attn_scores``.

            - feature (Tensor): Shape (N, T, E). Raw visual features for
              language decoder.
            - logits (Tensor): Shape (N, T, C). The raw logits for
              characters.
            - attn_scores (Tensor): Shape (N, T, H, W). Intermediate result
              for vision-language aligner.
        """
        return self.forward_train(
            feat, out_enc=out_enc, data_samples=data_samples)

    def _encoder_layer(self,
                       in_channels: int,
                       out_channels: int,
                       kernel_size: int = 3,
                       stride: int = 2,
                       padding: int = 1) -> nn.Sequential:
        """Generate encoder layer.

        Args:
            in_channels (int): Input channels.
            out_channels (int): Output channels.
            kernel_size (int, optional): Kernel size. Defaults to 3.
            stride (int, optional): Stride. Defaults to 2.
            padding (int, optional): Padding. Defaults to 1.

        Returns:
            nn.Sequential: Encoder layer.
        """
        return ConvModule(
            in_channels,
            out_channels,
            kernel_size=kernel_size,
            stride=stride,
            padding=padding,
            norm_cfg=dict(type='BN'),
            act_cfg=dict(type='ReLU'))

    def _decoder_layer(self,
                       in_channels: int,
                       out_channels: int,
                       kernel_size: int = 3,
                       stride: int = 1,
                       padding: int = 1,
                       mode: str = 'nearest',
                       scale_factor: Optional[int] = None,
                       size: Optional[Tuple[int, int]] = None):
        """Generate decoder layer.

        Args:
            in_channels (int): Input channels.
            out_channels (int): Output channels.
            kernel_size (int): Kernel size. Defaults to 3.
            stride (int): Stride. Defaults to 1.
            padding (int): Padding. Defaults to 1.
            mode (str): Interpolation mode. Defaults to 'nearest'.
            scale_factor (int, optional): Scale factor for upsampling.
            size (Tuple[int, int], optional): Output size. Defaults to None.
        """
        align_corners = None if mode == 'nearest' else True
        return nn.Sequential(
            nn.Upsample(
                size=size,
                scale_factor=scale_factor,
                mode=mode,
                align_corners=align_corners),
            ConvModule(
                in_channels,
                out_channels,
                kernel_size=kernel_size,
                stride=stride,
                padding=padding,
                norm_cfg=dict(type='BN'),
                act_cfg=dict(type='ReLU')))



# Copyright (c) OpenMMLab. All rights reserved.
import os.path as osp
import re
from typing import List, Optional, Tuple

from mmocr.registry import DATA_GATHERERS
from mmocr.utils import list_files
from .base import BaseGatherer


@DATA_GATHERERS.register_module()
class PairGatherer(BaseGatherer):
    """Gather the dataset files. Specifically for the paired annotations. That
    is to say, each image has a corresponding annotation file. For example,

            img_1.jpg <---> gt_img_1.txt
            img_2.jpg <---> gt_img_2.txt
            img_3.jpg <---> gt_img_3.txt

    Args:
        img_suffixes (List[str]): File suffixes that used for searching.
        rule (Sequence): The rule for pairing the files. The first element is
            the matching pattern for the file, and the second element is the
            replacement pattern, which should be a regular expression. For
            example, to map the image name img_1.jpg to the annotation name
            gt_img_1.txt, the rule is
            [r'img_(\d+)\.([jJ][pP][gG])', r'gt_img_\1.txt'] # noqa: W605 E501

    Note: PairGatherer assumes that each split annotation file is in the
    correspond split directory. For example, all the train annotation files are
    in {ann_dir}/train.
    """

    def __init__(self,
                 img_suffixes: Optional[List[str]] = None,
                 rule: Optional[List[str]] = None,
                 **kwargs) -> None:
        super().__init__(**kwargs)
        self.rule = rule
        self.img_suffixes = img_suffixes
        # ann_dir = {ann_root}/{ann_dir}/{split}
        self.ann_dir = osp.join(self.ann_dir, self.split)

    def __call__(self) -> Tuple[List[str], List[str]]:
        """tuple(list, list): The list of image paths and the list of
        annotation paths."""

        img_list = list()
        ann_list = list()
        for img_path in list_files(self.img_dir, self.img_suffixes):
            if not re.match(self.rule[0], osp.basename(img_path)):
                continue
            ann_name = re.sub(self.rule[0], self.rule[1],
                              osp.basename(img_path))
            ann_path = osp.join(self.ann_dir, ann_name)
            img_list.append(img_path)
            ann_list.append(ann_path)

        return img_list, ann_list



_base_ = [
    'drrg_resnet50_fpn-unet_1200e_ctw1500.py',
]

load_from = None

_base_.model.backbone = dict(
    type='CLIPResNet',
    init_cfg=dict(
        type='Pretrained',
        checkpoint='https://download.openmmlab.com/'
        'mmocr/backbone/resnet50-oclip-7ba0c533.pth'))

param_scheduler = [
    dict(type='LinearLR', end=100, start_factor=0.001),
    dict(type='PolyLR', power=0.9, eta_min=1e-7, begin=100, end=1200),
]



_base_ = '_base_abinet-vision.py'

model = dict(
    decoder=dict(
        d_model=512,
        num_iters=3,
        language_decoder=dict(
            type='ABILanguageDecoder',
            d_model=512,
            n_head=8,
            d_inner=2048,
            n_layers=4,
            dropout=0.1,
            detach_tokens=True,
            use_self_attn=False,
        )), )



# Copyright (c) OpenMMLab. All rights reserved.
import argparse
import math
import os.path as osp

import mmcv
import mmengine
import numpy as np

from mmocr.utils import dump_ocr_data


def parse_args():
    parser = argparse.ArgumentParser(
        description='Generate training, validation and test set of IMGUR ')
    parser.add_argument('root_path', help='Root dir path of IMGUR')
    args = parser.parse_args()

    return args


def collect_imgur_info(root_path, annotation_filename, print_every=1000):

    annotation_path = osp.join(root_path, 'annotations', annotation_filename)
    if not osp.exists(annotation_path):
        raise Exception(
            f'{annotation_path} not exists, please check and try again.')

    annotation = mmengine.load(annotation_path)
    images = annotation['index_to_ann_map'].keys()
    img_infos = []
    for i, img_name in enumerate(images):
        if i >= 0 and i % print_every == 0:
            print(f'{i}/{len(images)}')

        img_path = osp.join(root_path, 'imgs', img_name + '.jpg')

        # Skip not exist images
        if not osp.exists(img_path):
            continue

        img = mmcv.imread(img_path, 'unchanged')

        # Skip broken images
        if img is None:
            continue

        img_info = dict(
            file_name=img_name + '.jpg',
            height=img.shape[0],
            width=img.shape[1])

        anno_info = []
        for ann_id in annotation['index_to_ann_map'][img_name]:
            ann = annotation['ann_id'][ann_id]

            # The original annotation is oriented rects [x, y, w, h, a]
            box = np.fromstring(
                ann['bounding_box'][1:-2], sep=',', dtype=float)
            quadrilateral = convert_oriented_box(box)

            xs, ys = quadrilateral[::2], quadrilateral[1::2]
            x = max(0, math.floor(min(xs)))
            y = max(0, math.floor(min(ys)))
            w = math.floor(max(xs)) - x
            h = math.floor(max(ys)) - y
            bbox = [x, y, w, h]
            segmentation = quadrilateral

            anno = dict(
                iscrowd=0,
                category_id=1,
                bbox=bbox,
                area=w * h,
                segmentation=[segmentation])
            anno_info.append(anno)
        img_info.update(anno_info=anno_info)
        img_infos.append(img_info)

    return img_infos


def convert_oriented_box(box):

    x_ctr, y_ctr, width, height, angle = box[:5]
    angle = -angle * math.pi / 180

    tl_x, tl_y, br_x, br_y = -width / 2, -height / 2, width / 2, height / 2
    rect = np.array([[tl_x, br_x, br_x, tl_x], [tl_y, tl_y, br_y, br_y]])
    R = np.array([[np.cos(angle), -np.sin(angle)],
                  [np.sin(angle), np.cos(angle)]])
    poly = R.dot(rect)
    x0, x1, x2, x3 = poly[0, :4] + x_ctr
    y0, y1, y2, y3 = poly[1, :4] + y_ctr
    poly = np.array([x0, y0, x1, y1, x2, y2, x3, y3], dtype=np.float32)
    poly = get_best_begin_point_single(poly)

    return poly.tolist()


def get_best_begin_point_single(coordinate):

    x1, y1, x2, y2, x3, y3, x4, y4 = coordinate
    xmin = min(x1, x2, x3, x4)
    ymin = min(y1, y2, y3, y4)
    xmax = max(x1, x2, x3, x4)
    ymax = max(y1, y2, y3, y4)
    combine = [[[x1, y1], [x2, y2], [x3, y3], [x4, y4]],
               [[x2, y2], [x3, y3], [x4, y4], [x1, y1]],
               [[x3, y3], [x4, y4], [x1, y1], [x2, y2]],
               [[x4, y4], [x1, y1], [x2, y2], [x3, y3]]]
    dst_coordinate = [[xmin, ymin], [xmax, ymin], [xmax, ymax], [xmin, ymax]]
    force = 100000000.0
    force_flag = 0
    for i in range(4):
        temp_force = cal_line_length(combine[i][0], dst_coordinate[0]) \
            + cal_line_length(combine[i][1], dst_coordinate[1]) \
            + cal_line_length(combine[i][2], dst_coordinate[2]) \
            + cal_line_length(combine[i][3], dst_coordinate[3])
        if temp_force < force:
            force = temp_force
            force_flag = i
    if force_flag != 0:
        pass

    return np.array(combine[force_flag]).reshape(8)


def cal_line_length(point1, point2):

    return math.sqrt(
        math.pow(point1[0] - point2[0], 2) +
        math.pow(point1[1] - point2[1], 2))


def main():
    args = parse_args()
    root_path = args.root_path

    for split in ['train', 'val', 'test']:
        print(f'Processing {split} set...')
        with mmengine.Timer(
                print_tmpl='It takes {}s to convert IMGUR annotation'):
            anno_infos = collect_imgur_info(
                root_path, f'imgur5k_annotations_{split}.json')
            dump_ocr_data(anno_infos,
                          osp.join(root_path, f'instances_{split}.json'),
                          'textdet')


if __name__ == '__main__':
    main()



# Copyright (c) OpenMMLab. All rights reserved.
import json
from typing import List, Tuple

import numpy as np

from mmocr.registry import DATA_PARSERS
from .base import BaseParser


@DATA_PARSERS.register_module()
class NAFAnnParser(BaseParser):
    """NAF dataset parser.

    The original annotation format of this dataset is stored in json files,
    which has the following keys that will be used here:
        - 'textBBs': List of text bounding box objects
            - 'poly_points': list of [x,y] pairs, the box corners going
                top-left,top-right,bottom-right,bottom-left
            - 'id': id of the textBB, used to match with the text
        - 'transcriptions': Dict of transcription objects, use the 'id' key
            to match with the textBB.

    Some special characters are used in the transcription:
    "«text»" indicates that "text" had a strikethrough
    "¿" indicates the transcriber could not read a character
    "§" indicates the whole line or word was illegible
    "" (empty string) is if the field was blank

    Args:
        ignore (list(str)): The text of the ignored instances. Default: ['#'].
        det (bool): Whether to parse the detection annotation. Default: True.
            If False, the parser will consider special case in NAF dataset
            where the transcription is not available.
    """

    def __init__(self,
                 ignore: List[str] = ['#'],
                 det: bool = True,
                 **kwargs) -> None:
        self.ignore = ignore
        self.det = det
        super().__init__(**kwargs)

    def parse_file(self, img_path: str, ann_path: str) -> Tuple:
        """Convert single annotation."""
        instances = list()
        for poly, text in self.loader(ann_path):
            instances.append(
                dict(poly=poly, text=text, ignore=text in self.ignore))

        return img_path, instances

    def loader(self, file_path: str) -> str:
        """Load the annotation of the NAF dataset.

        Args:
            file_path (str): Path to the json file

        Retyrb:
            str: Complete annotation of the json file
        """
        with open(file_path, 'r') as f:
            data = json.load(f)

        # 'textBBs' contains the printed texts of the table while 'fieldBBs'
        #  contains the text filled by human.
        for box_type in ['textBBs', 'fieldBBs']:
            if not self.det:
                # 'textBBs' is only used for detection task.
                if box_type == 'textBBs':
                    continue
            for anno in data[box_type]:
                # Skip blanks
                if self.det:
                    if box_type == 'fieldBBs':
                        if anno['type'] == 'blank':
                            continue
                    poly = np.array(anno['poly_points']).reshape(
                        1, 8)[0].tolist()
                    # Since detection task only need poly, we can skip the
                    # transcription part that can be empty.
                    text = None
                else:
                    # For tasks that need transcription, NAF dataset has
                    # serval special cases:
                    # 1. The transcription for the whole image is not
                    # available.
                    # 2. The transcription for the certain text is not
                    # available.
                    # 3. If the length of the transcription is 0, it should
                    # be ignored.
                    if 'transcriptions' not in data.keys():
                        break
                    if anno['id'] not in data['transcriptions'].keys():
                        continue
                    text = data['transcriptions'][anno['id']]
                    text = text.strip(
                        '\u202a')  # Remove unicode control character
                    text = text.replace('»', '').replace(
                        '«', '')  # Remove strikethrough flag
                    if len(text) == 0:
                        continue
                    poly = np.array(anno['poly_points']).reshape(
                        1, 8)[0].tolist()
                yield poly, text



_base_ = [
    '../_base_/datasets/icdar2015.py',
    '../_base_/default_runtime.py',
    '../_base_/schedules/schedule_adam_600e.py',
    '_base_panet_resnet18_fpem-ffm.py',
]

default_hooks = dict(checkpoint=dict(type='CheckpointHook', interval=20), )

# dataset settings
icdar2015_textdet_train = _base_.icdar2015_textdet_train
icdar2015_textdet_test = _base_.icdar2015_textdet_test
# pipeline settings
icdar2015_textdet_train.pipeline = _base_.train_pipeline
icdar2015_textdet_test.pipeline = _base_.test_pipeline

train_dataloader = dict(
    batch_size=64,
    num_workers=8,
    persistent_workers=True,
    sampler=dict(type='DefaultSampler', shuffle=True),
    dataset=icdar2015_textdet_train)
val_dataloader = dict(
    batch_size=1,
    num_workers=4,
    persistent_workers=True,
    sampler=dict(type='DefaultSampler', shuffle=False),
    dataset=icdar2015_textdet_test)
test_dataloader = val_dataloader

val_evaluator = dict(
    type='HmeanIOUMetric', pred_score_thrs=dict(start=0.3, stop=1, step=0.05))
test_evaluator = val_evaluator

auto_scale_lr = dict(base_batch_size=64)



# Copyright (c) OpenMMLab. All rights reserved.
from mmengine.model import BaseModule

from mmocr.registry import MODELS


@MODELS.register_module()
class BaseEncoder(BaseModule):
    """Base Encoder class for text recognition."""

    def forward(self, feat, **kwargs):
        return feat



# Copyright (c) OpenMMLab. All rights reserved.
import glob
import os
import os.path as osp
import shutil
import ssl
import urllib.request as request
from typing import Dict, List, Optional, Tuple

from mmengine import mkdir_or_exist

from mmocr.registry import DATA_OBTAINERS
from mmocr.utils import check_integrity, is_archive

ssl._create_default_https_context = ssl._create_unverified_context


@DATA_OBTAINERS.register_module()
class NaiveDataObtainer:
    """A naive pipeline for obtaining dataset.

    download -> extract -> move

    Args:
        files (list[dict]): A list of file information.
        cache_path (str): The path to cache the downloaded files.
        data_root (str): The root path of the dataset. It is usually set auto-
            matically and users do not need to set it manually in config file
            in most cases.
        task (str): The task of the dataset. It is usually set automatically
            and users do not need to set it manually in config file
            in most cases.
    """

    def __init__(self, files: List[Dict], cache_path: str, data_root: str,
                 task: str) -> None:
        self.files = files
        self.cache_path = cache_path
        self.data_root = data_root
        self.task = task
        mkdir_or_exist(self.data_root)
        mkdir_or_exist(osp.join(self.data_root, f'{task}_imgs'))
        mkdir_or_exist(osp.join(self.data_root, 'annotations'))
        mkdir_or_exist(self.cache_path)

    def __call__(self):
        for file in self.files:
            save_name = file.get('save_name', None)
            url = file.get('url', None)
            md5 = file.get('md5', None)
            download_path = osp.join(
                self.cache_path,
                osp.basename(url) if save_name is None else save_name)
            # Download required files
            if not check_integrity(download_path, md5):
                self.download(url=url, dst_path=download_path)
            # Extract downloaded zip files to data root
            self.extract(src_path=download_path, dst_path=self.data_root)
            # Move & Rename dataset files
            if 'mapping' in file:
                self.move(mapping=file['mapping'])
        self.clean()

    def download(self, url: Optional[str], dst_path: str) -> None:
        """Download file from given url with progress bar.

        Args:
            url (str): The url to download the file.
            dst_path (str): The destination path to save the file.
        """

        def progress(down: float, block: float, size: float) -> None:
            """Show download progress.

            Args:
                down (float): Downloaded size.
                block (float): Block size.
                size (float): Total size of the file.
            """

            percent = min(100. * down * block / size, 100)
            file_name = osp.basename(dst_path)
            print(f'\rDownloading {file_name}: {percent:.2f}%', end='')

        if url is None and not osp.exists(dst_path):
            raise FileNotFoundError(
                'Direct url is not available for this dataset.'
                ' Please manually download the required files'
                ' following the guides.')

        if url.startswith('magnet'):
            raise NotImplementedError('Please use any BitTorrent client to '
                                      'download the following magnet link to '
                                      f'{osp.abspath(dst_path)} and '
                                      f'try again.\nLink: {url}')

        print('Downloading...')
        print(f'URL: {url}')
        print(f'Destination: {osp.abspath(dst_path)}')
        print('If you stuck here for a long time, please check your network, '
              'or manually download the file to the destination path and '
              'run the script again.')
        request.urlretrieve(url, dst_path, progress)
        print('')

    def extract(self,
                src_path: str,
                dst_path: str,
                delete: bool = False) -> None:
        """Extract zip/tar.gz files.

        Args:
            src_path (str): Path to the zip file.
            dst_path (str): Path to the destination folder.
            delete (bool, optional): Whether to delete the zip file. Defaults
                to False.
        """
        if not is_archive(src_path):
            # Copy the file to the destination folder if it is not a zip
            if osp.isfile(src_path):
                shutil.copy(src_path, dst_path)
            else:
                shutil.copytree(src_path, dst_path)
            return

        zip_name = osp.basename(src_path).split('.')[0]
        if dst_path is None:
            dst_path = osp.join(osp.dirname(src_path), zip_name)
        else:
            dst_path = osp.join(dst_path, zip_name)

        extracted = False
        if osp.exists(dst_path):
            name = set(os.listdir(dst_path))
            if '.finish' in name:
                extracted = True
            elif '.finish' not in name and len(name) > 0:
                while True:
                    c = input(f'{dst_path} already exists when extracting '
                              '{zip_name}, unzip again? (y/N) ') or 'N'
                    if c.lower() in ['y', 'n']:
                        extracted = c == 'n'
                        break
        if extracted:
            open(osp.join(dst_path, '.finish'), 'w').close()
            print(f'{zip_name} has been extracted. Skip')
            return
        mkdir_or_exist(dst_path)
        print(f'Extracting: {osp.basename(src_path)}')
        if src_path.endswith('.zip'):
            try:
                import zipfile
            except ImportError:
                raise ImportError(
                    'Please install zipfile by running "pip install zipfile".')
            with zipfile.ZipFile(src_path, 'r') as zip_ref:
                zip_ref.extractall(dst_path)
        elif src_path.endswith('.tar.gz') or src_path.endswith('.tar'):
            if src_path.endswith('.tar.gz'):
                mode = 'r:gz'
            elif src_path.endswith('.tar'):
                mode = 'r:'
            try:
                import tarfile
            except ImportError:
                raise ImportError(
                    'Please install tarfile by running "pip install tarfile".')
            with tarfile.open(src_path, mode) as tar_ref:
                tar_ref.extractall(dst_path)

        open(osp.join(dst_path, '.finish'), 'w').close()
        if delete:
            os.remove(src_path)

    def move(self, mapping: List[Tuple[str, str]]) -> None:
        """Rename and move dataset files one by one.

        Args:
            mapping (List[Tuple[str, str]]): A list of tuples, each
            tuple contains the source file name and the destination file name.
        """
        for src, dst in mapping:
            src = osp.join(self.data_root, src)
            dst = osp.join(self.data_root, dst)

            if '*' in src:
                mkdir_or_exist(dst)
                for f in glob.glob(src):
                    if not osp.exists(
                            osp.join(dst, osp.relpath(f, self.data_root))):
                        shutil.move(f, dst)

            elif osp.exists(src) and not osp.exists(dst):
                mkdir_or_exist(osp.dirname(dst))
                shutil.move(src, dst)

    def clean(self) -> None:
        """Remove empty dirs."""
        for root, dirs, files in os.walk(self.data_root, topdown=False):
            if not files and not dirs:
                os.rmdir(root)



# Copyright (c) OpenMMLab. All rights reserved.
import functools
import operator
from typing import Dict, List, Tuple, Union

import cv2
import numpy as np
import torch
from mmengine.structures import InstanceData
from numpy import ndarray

from mmocr.registry import MODELS
from mmocr.structures import TextDetDataSample
from .base import BaseTextDetPostProcessor


class Node:
    """A simple graph node.

    Args:
        ind (int): The index of the node.
    """

    def __init__(self, ind: int) -> None:
        self.__ind = ind
        self.__links = set()

    @property
    def ind(self) -> int:
        """Current node index."""
        return self.__ind

    @property
    def links(self) -> set:
        """A set of links."""
        return set(self.__links)

    def add_link(self, link_node: 'Node') -> None:
        """Add a link to the node.

        Args:
            link_node (Node): The link node.
        """
        self.__links.add(link_node)
        link_node.__links.add(self)


@MODELS.register_module()
class DRRGPostprocessor(BaseTextDetPostProcessor):
    """Merge text components and construct boundaries of text instances.

    Args:
        link_thr (float): The edge score threshold. Defaults to 0.8.
        edge_len_thr (int or float): The edge length threshold. Defaults to 50.
        rescale_fields (list[str]): The bbox/polygon field names to
            be rescaled. If None, no rescaling will be performed. Defaults to
            [polygons'].
    """

    def __init__(self,
                 link_thr: float = 0.8,
                 edge_len_thr: Union[int, float] = 50.,
                 rescale_fields=['polygons'],
                 **kwargs) -> None:
        super().__init__(rescale_fields=rescale_fields)
        assert isinstance(link_thr, float)
        assert isinstance(edge_len_thr, (int, float))
        self.link_thr = link_thr
        self.edge_len_thr = edge_len_thr

    def get_text_instances(self, pred_results: Tuple[ndarray, ndarray,
                                                     ndarray],
                           data_sample: TextDetDataSample
                           ) -> TextDetDataSample:
        """Get text instance predictions of one image.

        Args:
            pred_result (tuple(ndarray, ndarray, ndarray)): Prediction results
                edge, score and text_comps. Each of shape
                :math:`(N_{edges}, 2)`, :math:`(N_{edges},)` and
                :math:`(M, 9)`, respectively.
            data_sample (TextDetDataSample): Datasample of an image.

        Returns:
            TextDetDataSample: The original dataSample with predictions filled
            in. Polygons and results are saved in
            ``TextDetDataSample.pred_instances.polygons``. The confidence
            scores are saved in ``TextDetDataSample.pred_instances.scores``.
        """

        data_sample.pred_instances = InstanceData()
        polys = []
        scores = []

        pred_edges, pred_scores, text_comps = pred_results

        if pred_edges is not None:
            assert len(pred_edges) == len(pred_scores)
            assert text_comps.ndim == 2
            assert text_comps.shape[1] == 9

            vertices, score_dict = self._graph_propagation(
                pred_edges, pred_scores, text_comps)
            clusters = self._connected_components(vertices, score_dict)
            pred_labels = self._clusters2labels(clusters, text_comps.shape[0])
            text_comps, pred_labels = self._remove_single(
                text_comps, pred_labels)
            polys, scores = self._comps2polys(text_comps, pred_labels)

        data_sample.pred_instances.polygons = polys
        data_sample.pred_instances.scores = torch.FloatTensor(scores)

        return data_sample

    def split_results(self, pred_results: Tuple[ndarray, ndarray,
                                                ndarray]) -> List[Tuple]:
        """Split batched elements in pred_results along the first dimension
        into ``batch_num`` sub-elements and regather them into a list of dicts.

        However, DRRG only outputs one batch at inference time, so this
        function is a no-op.
        """
        return [pred_results]

    def _graph_propagation(self, edges: ndarray, scores: ndarray,
                           text_comps: ndarray) -> Tuple[List[Node], Dict]:
        """Propagate edge score information and construct graph. This code was
        partially adapted from https://github.com/GXYM/DRRG licensed under the
        MIT license.

        Args:
            edges (ndarray): The edge array of shape N * 2, each row is a node
                index pair that makes up an edge in graph.
            scores (ndarray): The edge score array.
            text_comps (ndarray): The text components.

        Returns:
            tuple(vertices, score_dict):

            - vertices (list[Node]): The Nodes in graph.
            - score_dict (dict): The edge score dict.
        """
        assert edges.ndim == 2
        assert edges.shape[1] == 2
        assert edges.shape[0] == scores.shape[0]
        assert text_comps.ndim == 2

        edges = np.sort(edges, axis=1)
        score_dict = {}
        for i, edge in enumerate(edges):
            if text_comps is not None:
                box1 = text_comps[edge[0], :8].reshape(4, 2)
                box2 = text_comps[edge[1], :8].reshape(4, 2)
                center1 = np.mean(box1, axis=0)
                center2 = np.mean(box2, axis=0)
                distance = np.linalg.norm(center1 - center2)
                if distance > self.edge_len_thr:
                    scores[i] = 0
            if (edge[0], edge[1]) in score_dict:
                score_dict[edge[0], edge[1]] = 0.5 * (
                    score_dict[edge[0], edge[1]] + scores[i])
            else:
                score_dict[edge[0], edge[1]] = scores[i]

        nodes = np.sort(np.unique(edges.flatten()))
        mapping = -1 * np.ones((np.max(nodes) + 1), dtype=int)
        mapping[nodes] = np.arange(nodes.shape[0])
        order_inds = mapping[edges]
        vertices = [Node(node) for node in nodes]
        for ind in order_inds:
            vertices[ind[0]].add_link(vertices[ind[1]])

        return vertices, score_dict

    def _connected_components(self, nodes: List[Node],
                              score_dict: Dict) -> List[List[Node]]:
        """Conventional connected components searching. This code was partially
        adapted from https://github.com/GXYM/DRRG licensed under the MIT
        license.

        Args:
            nodes (list[Node]): The list of Node objects.
            score_dict (dict): The edge score dict.

        Returns:
            List[list[Node]]: The clustered Node objects.
        """
        assert isinstance(nodes, list)
        assert all([isinstance(node, Node) for node in nodes])
        assert isinstance(score_dict, dict)

        clusters = []
        nodes = set(nodes)
        while nodes:
            node = nodes.pop()
            cluster = {node}
            node_queue = [node]
            while node_queue:
                node = node_queue.pop(0)
                neighbors = {
                    neighbor
                    for neighbor in node.links if score_dict[tuple(
                        sorted([node.ind, neighbor.ind]))] >= self.link_thr
                }
                neighbors.difference_update(cluster)
                nodes.difference_update(neighbors)
                cluster.update(neighbors)
                node_queue.extend(neighbors)
            clusters.append(list(cluster))
        return clusters

    def _clusters2labels(self, clusters: List[List[Node]],
                         num_nodes: int) -> ndarray:
        """Convert clusters of Node to text component labels. This code was
        partially adapted from https://github.com/GXYM/DRRG licensed under the
        MIT license.

        Args:
            clusters (List[list[Node]]): The clusters of Node objects.
            num_nodes (int): The total node number of graphs in an image.

        Returns:
            ndarray: The node label array.
        """
        assert isinstance(clusters, list)
        assert all([isinstance(cluster, list) for cluster in clusters])
        assert all([
            isinstance(node, Node) for cluster in clusters for node in cluster
        ])
        assert isinstance(num_nodes, int)

        node_labels = np.zeros(num_nodes)
        for cluster_ind, cluster in enumerate(clusters):
            for node in cluster:
                node_labels[node.ind] = cluster_ind
        return node_labels

    def _remove_single(self, text_comps: ndarray,
                       comp_pred_labels: ndarray) -> Tuple[ndarray, ndarray]:
        """Remove isolated text components. This code was partially adapted
        from https://github.com/GXYM/DRRG licensed under the MIT license.

        Args:
            text_comps (ndarray): The text components.
            comp_pred_labels (ndarray): The clustering labels of text
                components.

        Returns:
            tuple(filtered_text_comps, comp_pred_labels):

            - filtered_text_comps (ndarray): The text components with isolated
              ones removed.
            - comp_pred_labels (ndarray): The clustering labels with labels of
              isolated text components removed.
        """
        assert text_comps.ndim == 2
        assert text_comps.shape[0] == comp_pred_labels.shape[0]

        single_flags = np.zeros_like(comp_pred_labels)
        pred_labels = np.unique(comp_pred_labels)
        for label in pred_labels:
            current_label_flag = (comp_pred_labels == label)
            if np.sum(current_label_flag) == 1:
                single_flags[np.where(current_label_flag)[0][0]] = 1
        keep_ind = [
            i for i in range(len(comp_pred_labels)) if not single_flags[i]
        ]
        filtered_text_comps = text_comps[keep_ind, :]
        filtered_labels = comp_pred_labels[keep_ind]

        return filtered_text_comps, filtered_labels

    def _comps2polys(self, text_comps: ndarray, comp_pred_labels: ndarray
                     ) -> Tuple[List[ndarray], List[float]]:
        """Construct text instance boundaries from clustered text components.
        This code was partially adapted from https://github.com/GXYM/DRRG
        licensed under the MIT license.

        Args:
            text_comps (ndarray): The text components.
            comp_pred_labels (ndarray): The clustering labels of text
                components.

        Returns:
            tuple(boundaries, scores):

            - boundaries (list[ndarray]): The predicted boundaries of text
              instances.
            - scores (list[float]): The boundary scores.
        """
        assert text_comps.ndim == 2
        assert len(text_comps) == len(comp_pred_labels)
        boundaries = []
        scores = []
        if len(text_comps) < 1:
            return boundaries, scores
        for cluster_ind in range(0, int(np.max(comp_pred_labels)) + 1):
            cluster_comp_inds = np.where(comp_pred_labels == cluster_ind)
            text_comp_boxes = text_comps[cluster_comp_inds, :8].reshape(
                (-1, 4, 2)).astype(np.int32)
            score = np.mean(text_comps[cluster_comp_inds, -1])

            if text_comp_boxes.shape[0] < 1:
                continue

            elif text_comp_boxes.shape[0] > 1:
                centers = np.mean(
                    text_comp_boxes, axis=1).astype(np.int32).tolist()
                shortest_path = self._min_connect_path(centers)
                text_comp_boxes = text_comp_boxes[shortest_path]
                top_line = np.mean(
                    text_comp_boxes[:, 0:2, :],
                    axis=1).astype(np.int32).tolist()
                bot_line = np.mean(
                    text_comp_boxes[:, 2:4, :],
                    axis=1).astype(np.int32).tolist()
                top_line, bot_line = self._fix_corner(top_line, bot_line,
                                                      text_comp_boxes[0],
                                                      text_comp_boxes[-1])
                boundary_points = top_line + bot_line[::-1]

            else:
                top_line = text_comp_boxes[0, 0:2, :].astype(np.int32).tolist()
                bot_line = text_comp_boxes[0, 2:4:-1, :].astype(
                    np.int32).tolist()
                boundary_points = top_line + bot_line

            boundary = [p for coord in boundary_points for p in coord]
            boundaries.append(np.array(boundary, dtype=np.float32))
            scores.append(score)

        return boundaries, scores

    def _norm2(self, point1: List[int], point2: List[int]) -> float:
        """Calculate the norm of two points."""
        return ((point1[0] - point2[0])**2 + (point1[1] - point2[1])**2)**0.5

    def _min_connect_path(self, points: List[List[int]]) -> List[List[int]]:
        """Find the shortest path to traverse all points. This code was
        partially adapted from https://github.com/GXYM/DRRG licensed under the
        MIT license.

        Args:
            points(List[list[int]]): The point sequence
                [[x0, y0], [x1, y1], ...].

        Returns:
            List[list[int]]: The shortest index path.
        """
        assert isinstance(points, list)
        assert all([isinstance(point, list) for point in points])
        assert all(
            [isinstance(coord, int) for point in points for coord in point])

        points_queue = points.copy()
        shortest_path = []
        current_edge = [[], []]

        edge_dict0 = {}
        edge_dict1 = {}
        current_edge[0] = points_queue[0]
        current_edge[1] = points_queue[0]
        points_queue.remove(points_queue[0])
        while points_queue:
            for point in points_queue:
                length0 = self._norm2(point, current_edge[0])
                edge_dict0[length0] = [point, current_edge[0]]
                length1 = self._norm2(current_edge[1], point)
                edge_dict1[length1] = [current_edge[1], point]
            key0 = min(edge_dict0.keys())
            key1 = min(edge_dict1.keys())

            if key0 <= key1:
                start = edge_dict0[key0][0]
                end = edge_dict0[key0][1]
                shortest_path.insert(0,
                                     [points.index(start),
                                      points.index(end)])
                points_queue.remove(start)
                current_edge[0] = start
            else:
                start = edge_dict1[key1][0]
                end = edge_dict1[key1][1]
                shortest_path.append([points.index(start), points.index(end)])
                points_queue.remove(end)
                current_edge[1] = end

            edge_dict0 = {}
            edge_dict1 = {}

        shortest_path = functools.reduce(operator.concat, shortest_path)
        shortest_path = sorted(set(shortest_path), key=shortest_path.index)

        return shortest_path

    def _in_contour(self, contour: ndarray, point: ndarray) -> bool:
        """Whether a point is in a contour."""
        x, y = point
        return cv2.pointPolygonTest(contour, (int(x), int(y)), False) > 0.5

    def _fix_corner(self, top_line: List[List[int]], btm_line: List[List[int]],
                    start_box: ndarray, end_box: ndarray
                    ) -> Tuple[List[List[int]], List[List[int]]]:
        """Add corner points to predicted side lines. This code was partially
        adapted from https://github.com/GXYM/DRRG licensed under the MIT
        license.

        Args:
            top_line (List[list[int]]): The predicted top sidelines of text
                instance.
            btm_line (List[list[int]]): The predicted bottom sidelines of text
                instance.
            start_box (ndarray): The first text component box.
            end_box (ndarray): The last text component box.

        Returns:
            tuple(top_line, bot_line):

            - top_line (List[list[int]]): The top sidelines with corner point
              added.
            - bot_line (List[list[int]]): The bottom sidelines with corner
              point added.
        """
        assert isinstance(top_line, list)
        assert all(isinstance(point, list) for point in top_line)
        assert isinstance(btm_line, list)
        assert all(isinstance(point, list) for point in btm_line)
        assert start_box.shape == end_box.shape == (4, 2)

        contour = np.array(top_line + btm_line[::-1])
        start_left_mid = (start_box[0] + start_box[3]) / 2
        start_right_mid = (start_box[1] + start_box[2]) / 2
        end_left_mid = (end_box[0] + end_box[3]) / 2
        end_right_mid = (end_box[1] + end_box[2]) / 2
        if not self._in_contour(contour, start_left_mid):
            top_line.insert(0, start_box[0].tolist())
            btm_line.insert(0, start_box[3].tolist())
        elif not self._in_contour(contour, start_right_mid):
            top_line.insert(0, start_box[1].tolist())
            btm_line.insert(0, start_box[2].tolist())
        if not self._in_contour(contour, end_left_mid):
            top_line.append(end_box[0].tolist())
            btm_line.append(end_box[3].tolist())
        elif not self._in_contour(contour, end_right_mid):
            top_line.append(end_box[1].tolist())
            btm_line.append(end_box[2].tolist())
        return top_line, btm_line



mjsynth_textrecog_data_root = 'data/mjsynth'

mjsynth_textrecog_train = dict(
    type='OCRDataset',
    data_root=mjsynth_textrecog_data_root,
    ann_file='textrecog_train.json',
    pipeline=None)

mjsynth_sub_textrecog_train = dict(
    type='OCRDataset',
    data_root=mjsynth_textrecog_data_root,
    ann_file='subset_textrecog_train.json',
    pipeline=None)



# Copyright (c) OpenMMLab. All rights reserved.
from typing import Dict, List, Optional, Tuple, Union

import torch
import torch.nn.functional as F
from mmengine.model import BaseModule
from torch import nn

from mmocr.registry import MODELS


class UpBlock(BaseModule):
    """Upsample block for DRRG and TextSnake.

    DRRG: `Deep Relational Reasoning Graph Network for Arbitrary Shape
    Text Detection <https://arxiv.org/abs/2003.07493>`_.

    TextSnake: `A Flexible Representation for Detecting Text of Arbitrary
    Shapes <https://arxiv.org/abs/1807.01544>`_.

    Args:
        in_channels (list[int]): Number of input channels at each scale. The
            length of the list should be 4.
        out_channels (int): The number of output channels.
        init_cfg (dict or list[dict], optional): Initialization configs.
            Defaults to None.
    """

    def __init__(self,
                 in_channels: int,
                 out_channels: int,
                 init_cfg: Optional[Union[Dict, List[Dict]]] = None) -> None:
        super().__init__(init_cfg=init_cfg)

        assert isinstance(in_channels, int)
        assert isinstance(out_channels, int)

        self.conv1x1 = nn.Conv2d(
            in_channels, in_channels, kernel_size=1, stride=1, padding=0)
        self.conv3x3 = nn.Conv2d(
            in_channels, out_channels, kernel_size=3, stride=1, padding=1)
        self.deconv = nn.ConvTranspose2d(
            out_channels, out_channels, kernel_size=4, stride=2, padding=1)

    def forward(self, x: torch.Tensor) -> torch.Tensor:
        """Forward propagation."""
        x = F.relu(self.conv1x1(x))
        x = F.relu(self.conv3x3(x))
        x = self.deconv(x)
        return x


@MODELS.register_module()
class FPN_UNet(BaseModule):
    """The class for implementing DRRG and TextSnake U-Net-like FPN.

    DRRG: `Deep Relational Reasoning Graph Network for Arbitrary Shape
    Text Detection <https://arxiv.org/abs/2003.07493>`_.

    TextSnake: `A Flexible Representation for Detecting Text of Arbitrary
    Shapes <https://arxiv.org/abs/1807.01544>`_.

    Args:
        in_channels (list[int]): Number of input channels at each scale. The
            length of the list should be 4.
        out_channels (int): The number of output channels.
        init_cfg (dict or list[dict], optional): Initialization configs.
    """

    def __init__(
        self,
        in_channels: int,
        out_channels: int,
        init_cfg: Optional[Union[Dict, List[Dict]]] = dict(
            type='Xavier',
            layer=['Conv2d', 'ConvTranspose2d'],
            distribution='uniform')
    ) -> None:
        super().__init__(init_cfg=init_cfg)

        assert len(in_channels) == 4
        assert isinstance(out_channels, int)

        blocks_out_channels = [out_channels] + [
            min(out_channels * 2**i, 256) for i in range(4)
        ]
        blocks_in_channels = [blocks_out_channels[1]] + [
            in_channels[i] + blocks_out_channels[i + 2] for i in range(3)
        ] + [in_channels[3]]

        self.up4 = nn.ConvTranspose2d(
            blocks_in_channels[4],
            blocks_out_channels[4],
            kernel_size=4,
            stride=2,
            padding=1)
        self.up_block3 = UpBlock(blocks_in_channels[3], blocks_out_channels[3])
        self.up_block2 = UpBlock(blocks_in_channels[2], blocks_out_channels[2])
        self.up_block1 = UpBlock(blocks_in_channels[1], blocks_out_channels[1])
        self.up_block0 = UpBlock(blocks_in_channels[0], blocks_out_channels[0])

    def forward(self, x: List[Union[torch.Tensor,
                                    Tuple[torch.Tensor]]]) -> torch.Tensor:
        """
        Args:
            x (list[Tensor] | tuple[Tensor]): A list of four tensors of shape
                :math:`(N, C_i, H_i, W_i)`, representing C2, C3, C4, C5
                features respectively. :math:`C_i` should matches the number in
                ``in_channels``.

        Returns:
            Tensor: Shape :math:`(N, C, H, W)` where :math:`H=4H_0` and
            :math:`W=4W_0`.
        """
        c2, c3, c4, c5 = x

        x = F.relu(self.up4(c5))

        c4 = F.interpolate(
            c4, size=x.shape[2:], mode='bilinear', align_corners=True)
        x = torch.cat([x, c4], dim=1)
        x = F.relu(self.up_block3(x))

        c3 = F.interpolate(
            c3, size=x.shape[2:], mode='bilinear', align_corners=True)
        x = torch.cat([x, c3], dim=1)
        x = F.relu(self.up_block2(x))

        c2 = F.interpolate(
            c2, size=x.shape[2:], mode='bilinear', align_corners=True)
        x = torch.cat([x, c2], dim=1)
        x = F.relu(self.up_block1(x))

        x = self.up_block0(x)
        # the output should be of the same height and width as backbone input
        return x



_base_ = ['mmdet::_base_/models/mask-rcnn_r50_fpn.py']

mask_rcnn = _base_.pop('model')
# Adapt Mask R-CNN model to OCR task
mask_rcnn.update(
    dict(
        data_preprocessor=dict(pad_mask=False),
        rpn_head=dict(
            anchor_generator=dict(
                scales=[4], ratios=[0.17, 0.44, 1.13, 2.90, 7.46])),
        roi_head=dict(
            bbox_head=dict(num_classes=1),
            mask_head=dict(num_classes=1),
        )))

model = dict(type='MMDetWrapper', text_repr_type='poly', cfg=mask_rcnn)

train_pipeline = [
    dict(type='LoadImageFromFile', color_type='color_ignore_orientation'),
    dict(
        type='LoadOCRAnnotations',
        with_polygon=True,
        with_bbox=True,
        with_label=True,
    ),
    dict(
        type='TorchVisionWrapper',
        op='ColorJitter',
        brightness=32.0 / 255,
        saturation=0.5,
        contrast=0.5),
    dict(
        type='RandomResize',
        scale=(640, 640),
        ratio_range=(1.0, 4.125),
        keep_ratio=True),
    dict(type='RandomFlip', prob=0.5),
    dict(type='TextDetRandomCrop', target_size=(640, 640)),
    dict(type='MMOCR2MMDet', poly2mask=True),
    dict(
        type='mmdet.PackDetInputs',
        meta_keys=('img_path', 'ori_shape', 'img_shape', 'flip',
                   'scale_factor', 'flip_direction'))
]

test_pipeline = [
    dict(type='LoadImageFromFile', color_type='color_ignore_orientation'),
    dict(type='Resize', scale=(1920, 1920), keep_ratio=True),
    dict(
        type='LoadOCRAnnotations',
        with_polygon=True,
        with_bbox=True,
        with_label=True),
    dict(
        type='PackTextDetInputs',
        meta_keys=('img_path', 'ori_shape', 'img_shape', 'scale_factor'))
]



wildreceipt_openset_data_root = 'data/wildreceipt/'

wildreceipt_openset_train = dict(
    type='WildReceiptDataset',
    data_root=wildreceipt_openset_data_root,
    metainfo=dict(category=[
        dict(id=0, name='bg'),
        dict(id=1, name='key'),
        dict(id=2, name='value'),
        dict(id=3, name='other')
    ]),
    ann_file='openset_train.txt',
    pipeline=None)

wildreceipt_openset_test = dict(
    type='WildReceiptDataset',
    data_root=wildreceipt_openset_data_root,
    metainfo=dict(category=[
        dict(id=0, name='bg'),
        dict(id=1, name='key'),
        dict(id=2, name='value'),
        dict(id=3, name='other')
    ]),
    ann_file='openset_test.txt',
    test_mode=True,
    pipeline=None)



# Copyright (c) OpenMMLab. All rights reserved.
from mmengine.model import BaseModule

from mmocr.registry import MODELS


@MODELS.register_module()
class BasePreprocessor(BaseModule):
    """Base Preprocessor class for text recognition."""

    def forward(self, x, **kwargs):
        return x
